angular.module('iOnRoad.controllers', [])

    .controller('IntroCtrl', ["$scope", "$state", "$http", "$cordovaOauth", "_Utils", "_Settings", "UsersService", function($scope, $state, $http, $cordovaOauth, _Utils, _Settings, UsersService) {
        /**
         * Convert an image
         * to a base64 url
         * @param  {String}   url
         * @param  {Function} callback
         * @param  {String}   [outputFormat=image/png]
         */
        function convertImgToBase64URL(url, callback, outputFormat){
            var img = new Image();
            img.crossOrigin = 'Anonymous';
            img.onload = function(){
                var canvas = document.createElement('CANVAS'),
                ctx = canvas.getContext('2d'), dataURL;
                canvas.height = this.height;
                canvas.width = this.width;
                ctx.drawImage(this, 0, 0);
                dataURL = canvas.toDataURL(outputFormat);
                callback(dataURL);
                canvas = null;
            };
            img.src = url;
        }

        function turnCloudBackupOn() {
            if (!_Settings.initialized()) {
                return;
            }

            _Settings.load();

            _Settings.putValue("cloud_backup", true, "boolean",
                function (result) {
                    console.log("Succesfully turned on cloud backup");
                }, function (error) {
                    console.log("Error turning on cloud backup");
                });
        }

        $scope.init = function () {
            $scope.eula = false;
            $scope.stepIndex = 1;
            $scope.stepCount = 8;
            $scope.stepDirection = "next";
            $scope.animating = true;
            $scope.dimensions = _Utils.getWindowDimensions();
        };

        $scope.start = function () {
            setTimeout(function() {
                $scope.$apply(function() {
                    $scope.eula = true;
                    $scope.stepIndex++;
                    $scope.stepDirection = "next";
                    $scope.animating = true;
                });
            }, 500);
        };

        $scope.start_no_login = function () {
            $scope.eula = true;
            $scope.stepIndex++;
            $scope.stepDirection = "next";
            $scope.animating = true;
        };

        $scope.prev = function (source) {
            $scope.stepIndex--;
            $scope.stepDirection = "prev";
            $scope.animating = true;
        };

        $scope.next = function (source) {
            if ($scope.Settings.is_first_app_run) {
                $scope.stepIndex++;
                $scope.stepDirection = "next";
                $scope.animating = true;
            }
            else {
                if ($scope.stepIndex == 7) {
                    $state.go("home");
                }
                else {
                    $scope.stepIndex++;
                    $scope.stepDirection = "next";
                    $scope.animating = true;
                }
            }
        };

        $scope.login_google = function () {
            $cordovaOauth.google("904608269177-r7bnkjsl7r6u56scus2li0ei6vdstu0e.apps.googleusercontent.com", ["email", "profile"]).then(function (result) {
                $http.get('https://www.googleapis.com/oauth2/v1/userinfo?alt=json&access_token='+result.access_token)
                    .success(function(data, status, headers, config) {
                        console.log(data, status, headers, config);

                        convertImgToBase64URL(data.picture, function(pictureBase64) {
                            // Save user data
                            UsersService.setUserInfo("com.google", data.email, pictureBase64, function () {
                                turnCloudBackupOn();

                                // move to the next screen
                                $scope.start();
                            });
                        });
                    })
                    .error(function(data, status, headers, config) {
                      console.log(data, status, headers, config);
                    });

            }, function (error) {
                console.log("Error -> " + error);

                _Settings.putValue("error_msg", "Error logging-in with Google", "string",
                    function (result) {
                        console.log("Successfully written to the settings file", key, value, result);
                    },
                    function (error) {
                        console.log("Error writing to the settings file", key, value, error);
                    });
            });
        };

        $scope.login_facebook = function () {
            $cordovaOauth.facebook("256026064442814", ["email", "public_profile", "user_about_me"]).then(function (result) {
                $http.get('https://graph.facebook.com/me?access_token='+result.access_token)
                    .success(function(data, status, headers, config) {
                        console.log(data, status, headers, config);

                        convertImgToBase64URL('https://graph.facebook.com/v2.4/'+data.id+'/picture?width=100&height=100', function(pictureBase64) {
                            // Save user data
                            UsersService.setUserInfo("com.facebook.auth.login", data.email, pictureBase64, function () {
                                turnCloudBackupOn();

                                // move to the next screen
                                $scope.start();
                            });
                        });
                    })
                    .error(function(data, status, headers, config) {
                      console.log(data, status, headers, config);
                    });

            }, function (error) {
                console.log("Error -> " + error);

                _Settings.putValue("error_msg", "Error logging-in with Facebook", "string",
                    function (result) {
                        console.log("Successfully written to the settings file", key, value, result);
                    },
                    function (error) {
                        console.log("Error writing to the settings file", key, value, error);
                    });
            });
        };
    }])

    .controller('LoginCtrl', ["$scope", "$state", "$http", "$cordovaOauth", "_Utils", "_Settings", "UsersService", function($scope, $state, $http, $cordovaOauth, _Utils, _Settings, UsersService) {
        /**
         * Convert an image
         * to a base64 url
         * @param  {String}   url
         * @param  {Function} callback
         * @param  {String}   [outputFormat=image/png]
         */
        function convertImgToBase64URL(url, callback, outputFormat){
            var img = new Image();
            img.crossOrigin = 'Anonymous';
            img.onload = function(){
                var canvas = document.createElement('CANVAS'),
                ctx = canvas.getContext('2d'), dataURL;
                canvas.height = this.height;
                canvas.width = this.width;
                ctx.drawImage(this, 0, 0);
                dataURL = canvas.toDataURL(outputFormat);
                callback(dataURL);
                canvas = null;
            };
            img.src = url;
        }

        function turnCloudBackupOn() {
            if (!_Settings.initialized()) {
                return;
            }

            _Settings.load();

            _Settings.putValue("cloud_backup", true, "boolean",
                function (result) {
                    console.log("Succesfully turned on cloud backup");
                }, function (error) {
                    console.log("Error turning on cloud backup");
                });
        }

        $scope.init = function () {

        };

        $scope.later = function() {
            $state.go($state.params.return_state);
        };

        $scope.login_google = function () {
            $cordovaOauth.google("904608269177-r7bnkjsl7r6u56scus2li0ei6vdstu0e.apps.googleusercontent.com", ["email", "profile"]).then(function (result) {
                $http.get('https://www.googleapis.com/oauth2/v1/userinfo?alt=json&access_token='+result.access_token)
                    .success(function(data, status, headers, config) {
                        console.log(data, status, headers, config);

                        convertImgToBase64URL(data.picture, function(pictureBase64) {
                            // Save user data
                            UsersService.setUserInfo("com.google", data.email, pictureBase64, function () {
                                turnCloudBackupOn();

                                // Move to the home screen
                                $state.go($state.params.return_state);
                            });
                        });
                    })
                    .error(function(data, status, headers, config) {
                      console.log(data, status, headers, config);
                    });

            }, function (error) {
                console.log("Error -> " + error);

                _Settings.putValue("error_msg", "Error logging-in with Google", "string",
                    function (result) {
                        console.log("Successfully written to the settings file", key, value, result);
                    },
                    function (error) {
                        console.log("Error writing to the settings file", key, value, error);
                    });
            });
        };

        $scope.login_facebook = function () {
            $cordovaOauth.facebook("256026064442814", ["email", "public_profile", "user_about_me"]).then(function (result) {
                $http.get('https://graph.facebook.com/me?access_token='+result.access_token)
                    .success(function(data, status, headers, config) {
                        console.log(data, status, headers, config);

                        convertImgToBase64URL('https://graph.facebook.com/v2.4/'+data.id+'/picture?width=100&height=100', function(pictureBase64) {
                            // Save user data
                            UsersService.setUserInfo("com.facebook.auth.login", data.email, pictureBase64, function () {
                                turnCloudBackupOn();

                                // Move to the home screen
                                $state.go($state.params.return_state);
                            });
                        });
                    })
                    .error(function(data, status, headers, config) {
                      console.log(data, status, headers, config);
                    });

            }, function (error) {
                console.log("Error -> " + error);

                _Settings.putValue("error_msg", "Error logging-in with Facebook", "string",
                    function (result) {
                        console.log("Successfully written to the settings file", key, value, result);
                    },
                    function (error) {
                        console.log("Error writing to the settings file", key, value, error);
                    });
            });
        };
    }])

    .controller('TestCtrl', ["$scope", "$timeout", "_Utils", "_Settings", "_SettingsLoader", function ($scope, $timeout, _Utils, _Settings, _SettingsLoader) {
        $scope.init = function () {
            $scope.eula = false;
            $scope.stepIndex = 1;
            $scope.stepCount = 4;
            $scope.stepDirection = "next";
            $scope.animating = true;
            $scope.dimensions = _Utils.getWindowDimensions();

            // Lets make sure the user can't switch back to the main
            // application while using in test at home
            _Settings.putValue("should_lock_orientation", true, "boolean",
                function (result) {
                    console.log("Succesfully locked orientation");
                }, function (error) {
                    console.log("Error writing locking orientation");
                });
        };

        $scope.prev = function (source) {
            if ($scope.stepIndex > 1) {
                $scope.stepIndex--;
                $scope.stepDirection = "prev";
                $scope.animating = true;
            }
        };

        $scope.next = function (source) {
            $scope.stepIndex++;
            $scope.stepDirection = "next";
            $scope.animating = true;
        };

        $scope.start = function () {
            // Allow orientation change again
            _Settings.putValue("should_lock_orientation", false, "boolean",
                function (result) {
                    console.log("Succesfully locked orientation");

                    // Start testing
                    _Settings.putValue("test_mode", true, "boolean",
                        function (result) {
                            console.log("Successfully written to the settings file", result);
                        },
                        function (error) {
                            console.log("Error writing to the settings file", error);
                        });

                }, function (error) {
                    console.log("Error writing locking orientation");
                });
        };
    }])

    .controller('HomeCtrl', ["$scope", "$stateParams", "$ionicHistory", "HomeService", "ParkingService", "_Settings", "_Utils", "_LatestRuns", "_ParkingInfo", "_IsParkingExists", "_ReminderInfo", "_UserInfo", "_SettingsData", "_IsPro", function ($scope, $stateParams, $ionicHistory, HomeService, ParkingService, _Settings, _Utils, _LatestRuns, _ParkingInfo, _IsParkingExists, _ReminderInfo, _UserInfo, _SettingsData, _IsPro) {

        $scope._LatestRuns = _LatestRuns;
        $scope._ParkingInfo = _ParkingInfo;
        $scope._IsParkingExists = _IsParkingExists;
        $scope._ReminderInfo = _ReminderInfo;
        $scope._UserInfo = _UserInfo;
        $scope._IsPro = _SettingsData[35].promise.$$state.value;

        function setMarker(map, icon, coor) {
            return new google.maps.Marker({
                position: coor,
                map: map,
                icon: icon,
                optimized: false
            });
        }

        function initializeMap() {
            if (typeof(google)==="undefined") {
                return;
            }

            var myLatlng = new google.maps.LatLng($scope.park.Latitude, $scope.park.Longitude);
            var mapOptions = {
                center: myLatlng,
                zoom: 14,
                mapTypeId: google.maps.MapTypeId.ROADMAP,
                panControl: false,
                zoomControl: false,
                mapTypeControl: false,
                scaleControl: false,
                streetViewControl: false,
                overviewMapControl: false,
                draggable: false,
                scrollwheel: false,
                maxZoom: 14,
                minZoom: 14
            };
            var map = new google.maps.Map(document.getElementById("map-park"),
                mapOptions);

            $scope.map = map;

            setMarker($scope.map,
                new google.maps.MarkerImage(img_dir + "icon-park.png",
                    new google.maps.Size(27, 38),
                    new google.maps.Point(0, 0),
                    new google.maps.Point(13, 25),
                    new google.maps.Size(27, 38)),
                new google.maps.LatLng($scope.park.Latitude, $scope.park.Longitude));
        }

        $scope.init = function () {
            $ionicHistory.clearHistory();

            // Initialize the users' info
            $scope.user = _UserInfo;

            // Initialize runs
            $scope.runs = $scope._LatestRuns;

            // Is parking exists?
            $scope.isParkingExists = $scope._IsParkingExists;

            // Initializie parking info
            if ($scope.isParkingExists) {
                $scope.park = $scope._ParkingInfo;
                initializeMap();
            }

            // $scope.$watch('loaded_remote_scripts', function(newValue, oldValue) {
            //     if ((oldValue != newValue) && (newValue === true)) {
            //         initializeMap();
            //     }
            // });

            // Initialize reminder info
            $scope.reminder = $scope._ReminderInfo;

            // Reset the orientation lock
            _Settings.putValue("should_lock_orientation", false, "boolean",
                function (result) {
                    console.log("Succesfully released orientation lock");
                }, function (error) {
                    console.log("Error while releasing orientation lock");
                });
        };
    }])

    .controller('MyDrivesCtrl', ["$scope", "$stateParams", "MyDrivesService", "_Utils", "_Settings", "_AllRuns", "_TotalDistance", "_TotalAlerts", "_TotalTime", function ($scope, $stateParams, MyDrivesService, _Utils, _Settings, _AllRuns, _TotalDistance, _TotalAlerts, _TotalTime) {
        $scope._AllRuns = _AllRuns;
        $scope._TotalDistance = _TotalDistance;
        $scope._TotalAlerts = _TotalAlerts;
        $scope._TotalTime = _TotalTime;

        $scope.init = function () {
            $scope.$on('$ionicView.beforeEnter', function () {
                $scope.totalDistance = 0;
                $scope.totalDistanceMetric = "";
                $scope.totalAlerts = 0;
                $scope.totalTime = 0;
                $scope.totalTimeMetric = "";

                // Initialize all runs
                $scope.runs = $scope._AllRuns;
                $scope.firstRunTime = $scope.getFirstRunTime();
                $scope.lastRunTime = $scope.getLastRunTime();
                if ($scope.firstRunTime != $scope.lastRunTime) {
                    $scope.runsTimespan = $scope.firstRunTime +" - "+$scope.lastRunTime;
                } else {
                    $scope.runsTimespan = $scope.firstRunTime;
                }

                // Initialize the total distance
                $scope.totalDistance = _Utils.getDistance($scope._TotalDistance);
                $scope.totalDistanceMetric = _Utils.getDistanceMetric($scope._TotalDistance, $scope.Settings.speed_units);

                // Initialize total alerts
                $scope.totalAlerts = $scope._TotalAlerts;

                // Initialize total time
                $scope.totalTime = _Utils.getDurationTime($scope._TotalTime);
                $scope.totalTimeMetric = _Utils.getDurationTimeMetric($scope._TotalTime);
            });
        };

        // Gets the start time of the first run
        $scope.getFirstRunTime = function () {
            return $scope.runs[$scope.runs.length - 1].Timestamp;
        };
        // Gets the start time of the last run
        $scope.getLastRunTime = function () {
            return $scope.runs[0].Timestamp;
        };
    }])

    .controller('DriveCtrl', ["$scope", "$state", "$stateParams", "$ionicHistory", "$ionicPopup", "DriveService", "RunsService", "_Utils", "_Settings", "_RunInfo", "_RouteInfo", "_Alerts", "_Delay", "_IsPro", function ($scope, $state, $stateParams, $ionicHistory, $ionicPopup, DriveService, RunsService, _Utils, _Settings, _RunInfo, _RouteInfo, _Alerts, _Delay, _IsPro) {
        $scope._RunInfo = _RunInfo;
        $scope._RouteInfo = _RouteInfo;
        $scope._Alerts = _Alerts;
        $scope._IsPro = _IsPro;

        /**
         * The CenterControl adds a control to the map that recenters the map on Chicago.
         * This constructor takes the control DIV as an argument.
         * @constructor
         */
        function CenterControl(controlDiv, map) {

            // Set CSS for the control border
            var controlUI = document.createElement('img');
            controlUI.src = "img/map/icon-center.png";
            controlUI.width = 32;
            controlUI.height = 32;
            controlUI.style.marginLeft = "10px";
            //controlUI.style.marginTop = "-38px";
            controlUI.style.display = "block";
            controlUI.style.opacity = 0.85;
            controlDiv.appendChild(controlUI);

            // Setup the click event listeners: simply set the map to
            // Chicago
            google.maps.event.addDomListener(controlUI, 'click', function () {
                $scope.map.setCenter(new google.maps.LatLng($scope.center.Latitude, $scope.center.Longitude));
                $scope.map.setZoom(14);
            });

        }

        // Initialize the map
        function initializeMap() {
            if (typeof(google)==="undefined") {
                return;
            }

            var myLatlng = new google.maps.LatLng(0, 0);
            var mapOptions = {
                center: myLatlng,
                zoom: 14,
                mapTypeId: google.maps.MapTypeId.ROADMAP,
                panControl: false,
                zoomControl: false,
                mapTypeControl: false,
                scaleControl: false,
                streetViewControl: false,
                overviewMapControl: false
            };
            var map = new google.maps.Map(document.getElementById("map-run"),
                mapOptions);
            $scope.mapCenter = myLatlng;
            $scope.map = map;

            // Resize
            document.getElementById("map-run").style.height = ($scope.dimensions.height-428 )+"px";
            google.maps.event.trigger(map, 'resize');

            // Create the DIV to hold the control and
            // call the CenterControl() constructor passing
            // in this DIV.
            var centerControlDiv = document.createElement('div');
            var centerControl = new CenterControl(centerControlDiv, map);

            centerControlDiv.index = 1;
            map.controls[google.maps.ControlPosition.LEFT_CENTER].push(centerControlDiv);

            updateMap();
        }

        function getTailgatingCoorColor(c) {
            return c.state;
        }

        function updateMap() {
            // First let's clear the map from all the elements
            // we added to it already
            if ($scope.mapData.activeMap.length > 0) {
                for (i = 0; i < $scope.mapData.activeMap.length; i++) {
                    clearMapElement($scope.mapData.activeMap[i]);
                }
            }
            // Which of the maps needs to be displayed?
            var source;
            var destination;
            var route;
            if ($state.includes("drive.collision")) {
                // Save the destination coordinates
                source = $scope.mapData.route[0];
                destination = $scope.mapData.route[$scope.mapData.route.length - 1];
                // Set the route
                route = [];
                for (i = 0; i < $scope.mapData.route.length; i++) {
                    var c = $scope.mapData.route[i];
                    route.push(new google.maps.LatLng(c.Latitude, c.Longitude));
                }
                runSnapToRoad(route, function (snappedCoordinates) {
                    var runPath = new google.maps.Polyline({
                        path: snappedCoordinates,
                        geodesic: true,
                        strokeColor: $scope.mapData.metadata.colors.route,
                        strokeOpacity: 1.0,
                        strokeWeight: 8,
                        map: $scope.map
                    });
                    $scope.mapData.activeMap.push(runPath);
                });
                // Set the source/destination markers
                var sourceMarker = setMarker($scope.map,
                    $scope.mapData.metadata.icons.start,
                    new google.maps.LatLng(source.Latitude, source.Longitude));
                $scope.mapData.activeMap.push(sourceMarker);
                var endMarker = new google.maps.Marker({
                    position: new google.maps.LatLng(destination.Latitude, destination.Longitude),
                    map: $scope.map,
                    icon: $scope.mapData.metadata.icons.end
                });
                $scope.mapData.activeMap.push(endMarker);
                // Set the collision markers
                for (i = 0; i < $scope.mapData.collisions.length; i++) {
                    var c = $scope.mapData.collisions[i];
                    var m = setMarker($scope.map,
                        $scope.mapData.metadata.collision.icons.alert,
                        new google.maps.LatLng(c.Latitude, c.Longitude));
                    m.metadata = c;
                    $scope.mapData.activeMap.push(m);
                    $scope.registerCollision(m);
                }
            } else if ($state.includes("drive.offroad")) {
                // Save the destination coordinates
                source = $scope.mapData.route[0];
                destination = $scope.mapData.route[$scope.mapData.route.length - 1];
                // Set the route
                route = [];
                for (i = 0; i < $scope.mapData.route.length; i++) {
                    var c = $scope.mapData.route[i];
                    route.push(new google.maps.LatLng(c.Latitude, c.Longitude));
                }
                runSnapToRoad(route, function (snappedCoordinates) {
                    var runPath = new google.maps.Polyline({
                        path: snappedCoordinates,
                        geodesic: true,
                        strokeColor: $scope.mapData.metadata.colors.route,
                        strokeOpacity: 1.0,
                        strokeWeight: 8,
                        map: $scope.map
                    });
                    $scope.mapData.activeMap.push(runPath);
                });
                // Set the source/destination markers
                var sourceMarker = setMarker($scope.map,
                    $scope.mapData.metadata.icons.start,
                    new google.maps.LatLng(source.Latitude, source.Longitude));
                $scope.mapData.activeMap.push(sourceMarker);
                var endMarker = setMarker($scope.map,
                    $scope.mapData.metadata.icons.end,
                    new google.maps.LatLng(destination.Latitude, destination.Longitude));
                $scope.mapData.activeMap.push(endMarker);
                // Set the offroad markers
                for (i = 0; i < $scope.mapData.offroad.length; i++) {
                    var c = $scope.mapData.offroad[i];
                    var m;
                    if (c.MaxValue > 0) {
                        m = setMarker($scope.map,
                            $scope.mapData.metadata.offroad.icons.alert_right,
                            new google.maps.LatLng(c.Latitude, c.Longitude));
                    }
                    else {
                        m = setMarker($scope.map,
                            $scope.mapData.metadata.offroad.icons.alert_left,
                            new google.maps.LatLng(c.Latitude, c.Longitude));
                    }
                    $scope.mapData.activeMap.push(m);
                }
            } else if ($state.includes("drive.speeding")) {
                // Save the destination coordinates
                source = $scope.mapData.route[0];
                destination = $scope.mapData.route[$scope.mapData.route.length - 1];
                // Set the route
                route = [];
                for (i = 0; i < $scope.mapData.route.length; i++) {
                    var c = $scope.mapData.route[i];
                    route.push(new google.maps.LatLng(c.Latitude, c.Longitude));
                }
                runSnapToRoad(route, function (snappedCoordinates) {
                    var runPath = new google.maps.Polyline({
                        path: snappedCoordinates,
                        geodesic: true,
                        strokeColor: $scope.mapData.metadata.colors.route,
                        strokeOpacity: 1.0,
                        strokeWeight: 8,
                        map: $scope.map
                    });
                    $scope.mapData.activeMap.push(runPath);
                });
                // Set the source/destination markers
                var sourceMarker = setMarker($scope.map,
                    $scope.mapData.metadata.icons.start,
                    new google.maps.LatLng(source.Latitude, source.Longitude));
                $scope.mapData.activeMap.push(sourceMarker);
                var endMarker = setMarker($scope.map,
                    $scope.mapData.metadata.icons.end,
                    new google.maps.LatLng(destination.Latitude, destination.Longitude));
                $scope.mapData.activeMap.push(endMarker);
                // Set the speeding markers
                for (i = 0; i < $scope.mapData.speeding.length; i++) {
                    var c = $scope.mapData.speeding[i];
                    if (c.SpeedSign) {
                        var speedSign = parseInt(c.SpeedSign.replace("USSpeed").replace("Speed"));
                        var speedActual = c.Speed;
                        var m;
                        if (speedActual < speedSign) {
                            m = setMarker($scope.map,
                                $scope.mapData.metadata.speeding.icons.alert_happy,
                                new google.maps.LatLng(c.Latitude, c.Longitude));
                        }
                        else {
                            m = setMarker($scope.map,
                                $scope.mapData.metadata.speeding.icons.alert_sad,
                                new google.maps.LatLng(c.Latitude, c.Longitude));
                        }
                        m.metadata = c;
                        $scope.mapData.activeMap.push(m);
                        $scope.registerSpeeding(m);
                    }
                }
            } else if ($state.includes("drive.tailgating")) {
                // Save the destination coordinates
                source = $scope.mapData.route[0];
                destination = $scope.mapData.route[$scope.mapData.route.length - 1];
                // Set the source/destination markers
                var sourceMarker = setMarker($scope.map,
                    $scope.mapData.metadata.icons.start,
                    new google.maps.LatLng(source.Latitude, source.Longitude));
                $scope.mapData.activeMap.push(sourceMarker);
                var endMarker = setMarker($scope.map,
                    $scope.mapData.metadata.icons.end,
                    new google.maps.LatLng(destination.Latitude, destination.Longitude));
                $scope.mapData.activeMap.push(endMarker);
                // Set the route
                route = [];
                var currTailgatingColor = getTailgatingCoorColor($scope.mapData.tailgating[0]);
                for (i = 0; i < $scope.mapData.tailgating.length; i++) {
                    var c = $scope.mapData.tailgating[i].coor;
                    var currColor = getTailgatingCoorColor($scope.mapData.tailgating[i]);
                    if (currColor != currTailgatingColor) {
                        var color;
                        var collision_alert_coor = null;
                        switch (currTailgatingColor) {
                            case 0: // none
                            {
                                color = $scope.mapData.metadata.colors.none;
                                break;
                            }
                            case 1: // red
                            {
                                $scope.mapData[$scope.mapData.length-1] = c;
                                color = $scope.mapData.metadata.colors.collision_route;
                                if (route.length >= 30) {
                                    collision_alert_coor = route[Math.floor(route.length / 2)];
                                }
                                break;
                            }
                            case 2: // yellow
                            {
                                color = $scope.mapData.metadata.colors.speeding_route;
                                break;
                            }
                            case 3: // green
                            {
                                color = $scope.mapData.metadata.colors.offroad_route;
                                break;
                            }
                        }

                        currTailgatingColor = currColor;

                        runSnapToRoad(route, function (snappedCoordinates, color) {
                            var segmentPath = new google.maps.Polyline({
                                path: snappedCoordinates,
                                geodesic: true,
                                strokeColor: color,
                                strokeOpacity: 1.0,
                                strokeWeight: 8,
                                map: $scope.map
                            });
                            $scope.mapData.activeMap.push(segmentPath);
                        }, color);
                        if (collision_alert_coor) {
                            var collisionAlert = setMarkerWithLabel($scope.map,
                                $scope.mapData.metadata.tailgating.icons.alert, collision_alert_coor);
                            $scope.mapData.activeMap.push(collisionAlert);
                        }
                        var temp = route[route.length-1];
                        route = [temp];
                    }
                    route.push(new google.maps.LatLng(c.Latitude, c.Longitude));
                }
                if (route.length > 0) {
                    var color;
                    switch (currTailgatingColor) {
                        case 0: // none
                        {
                            color = $scope.mapData.metadata.colors.none;
                            break;
                        }
                        case 1: // red
                        {
                            color = $scope.mapData.metadata.colors.collision_route;
                            break;
                        }
                        case 2: // yellow
                        {
                            color = $scope.mapData.metadata.colors.speeding_route;
                            break;
                        }
                        case 3: // green
                        {
                            color = $scope.mapData.metadata.colors.offroad_route;
                            break;
                        }
                    }
                    runSnapToRoad(route, function (snappedCoordinates, color) {
                        var segmentPath = new google.maps.Polyline({
                            path: snappedCoordinates,
                            geodesic: true,
                            strokeColor: color,
                            strokeOpacity: 1.0,
                            strokeWeight: 8,
                            map: $scope.map
                        });
                        $scope.mapData.activeMap.push(segmentPath);
                    }, color);
                }
            }
            if (!$scope.mapData.initialized) {

                $scope.mapData.initialized = true;

                // Center the map to the source coordinates
                var center = $scope.mapData.route[0];
                $scope.center = center;
                $scope.mapData.initialized = true;
                $scope.map.setCenter(new google.maps.LatLng(center.Latitude, center.Longitude));
                $scope.map.setZoom(14);
            }
        }

        function runSnapToRoad(path, callback, color) {
            if (callback) callback(path, color);
            //var result, other;
            //result = _.partition(path, function(n, index) {
            //    return (index % 10 == 0);
            //});
            //
            //other = _.slice(result[0],101,99999);
            //result = _.slice(result[0],0,100);
            //
            //var pathValues = [];
            //for (var i = 0; i < result.length; i++) {
            //    pathValues.push(path[i].toUrlValue());
            //}
            //
            //$http({
            //    url: 'https://roads.googleapis.com/v1/snapToRoads',
            //    method: "GET",
            //    params: {
            //        interpolate: true,
            //        key: "AIzaSyAWyADilRbdQqh40h3HLAbKLjGA1fS9Alw",
            //        path: pathValues.join('|')
            //    }
            //})
            //.success(function(data, status, headers, config) {
            //    snappedCoordinates = [];
            //    placeIdArray = [];
            //    for (var i = 0; i < data.snappedPoints.length; i++) {
            //        var latlng = new google.maps.LatLng(
            //            data.snappedPoints[i].location.latitude,
            //            data.snappedPoints[i].location.longitude);
            //        snappedCoordinates.push(latlng);
            //        placeIdArray.push(data.snappedPoints[i].placeId);
            //    }
            //    //if (callback) callback(snappedCoordinates, color);
            //    if (callback) callback(path, color);
            //})
            //.error(function(data, status, headers, config) {
            //    if (callback) callback(path, color)
            //});
            //
            //if (other.length > 0)
            //    runSnapToRoad(other, color, callback);
        }

        function setMarker(map, icon, coor) {
            return new google.maps.Marker({
                position: coor,
                map: map,
                icon: icon,
                optimized: false
            });
        }

        function setMarkerWithLabel(map, icon, coor) {
            return new MarkerWithLabel({
                position: coor,
                map: map,
                icon: icon,
                labelAnchor: new google.maps.Point(10, 10),
                labelClass: "sonar"
            });
        }

        function clearMapElement(marker) {
            marker.setMap(null);
        }

        $scope.init = function () {
            // Initialize data members
            $scope.route = [];

            // Initialize the charts
            $scope.myChartData = [
                {
                    value: 0, //$scope.totalCollisionAlerts,
                    color: "#f53721"
                },
                {
                    value: 0, //$scope.totalSpeedingAlerts,
                    color: "#00b9f1"
                },
                {
                    value: 0, //$scope.totalTailgatingAlerts,
                    color: "#ffd800"
                },
                {
                    value: 0, //$scope.totalOffroadAlerts,
                    color: "#01ecb2"
                },
                // {
                //     value: 100,
                //     color: "#b6bdc3"
                // }
            ];
            $scope.myChartOptions = {
                // Boolean - Whether to animate the chart
                animation: true,
                //Number - The percentage of the chart that we cut out of the middle
                percentageInnerCutout: 75, // This is 0 for Pie charts
                // Number - Number of animation steps
                animationSteps: 30,
                // String - Animation easing effect
                animationEasing: "easeOutQuart",
                // Boolean - If we should show the scale at all
                showScale: true,
                // Boolean - If we want to override with a hard coded scale
                scaleOverride: false,
                // ** Required if scaleOverride is true **
                // Number - The number of steps in a hard coded scale
                scaleSteps: null,
                // Number - The value jump in the hard coded scale
                scaleStepWidth: null,
                // Number - The scale starting value
                scaleStartValue: null,
                // String - Colour of the scale line
                scaleLineColor: "rgba(0,0,0,.1)",
                // Number - Pixel width of the scale line
                scaleLineWidth: 1,
                // Boolean - Whether to show labels on the scale
                scaleShowLabels: true,
                // Interpolated JS string - can access value
                scaleLabel: "<%=value%>",
                // Boolean - Whether the scale should stick to integers, not floats even if drawing space is there
                scaleIntegersOnly: true,
                // Boolean - Whether the scale should start at zero, or an order of magnitude down from the lowest value
                scaleBeginAtZero: false,
                // String - Scale label font declaration for the scale label
                scaleFontFamily: "'Helvetica Neue', 'Helvetica', 'Arial', sans-serif",
                // Number - Scale label font size in pixels
                scaleFontSize: 12,
                // String - Scale label font weight style
                scaleFontStyle: "normal",
                // String - Scale label font colour
                scaleFontColor: "#666",
                // Boolean - whether or not the chart should be responsive and resize when the browser does.
                responsive: false,
                // Boolean - Determines whether to draw tooltips on the canvas or not
                showTooltips: true,
                // Array - Array of string names to attach tooltip events
                tooltipEvents: ["mousemove", "touchstart", "touchmove"],
                // String - Tooltip background colour
                tooltipFillColor: "rgba(0,0,0,0.8)",
                // String - Tooltip label font declaration for the scale label
                tooltipFontFamily: "'Helvetica Neue', 'Helvetica', 'Arial', sans-serif",
                // Number - Tooltip label font size in pixels
                tooltipFontSize: 14,
                // String - Tooltip font weight style
                tooltipFontStyle: "normal",
                // String - Tooltip label font colour
                tooltipFontColor: "#fff",
                // String - Tooltip title font declaration for the scale label
                tooltipTitleFontFamily: "'Helvetica Neue', 'Helvetica', 'Arial', sans-serif",
                // Number - Tooltip title font size in pixels
                tooltipTitleFontSize: 14,
                // String - Tooltip title font weight style
                tooltipTitleFontStyle: "bold",
                // String - Tooltip title font colour
                tooltipTitleFontColor: "#fff",
                // Number - pixel width of padding around tooltip text
                tooltipYPadding: 6,
                // Number - pixel width of padding around tooltip text
                tooltipXPadding: 6,
                // Number - Size of the caret on the tooltip
                tooltipCaretSize: 8,
                // Number - Pixel radius of the tooltip border
                tooltipCornerRadius: 6,
                // Number - Pixel offset from point x to tooltip edge
                tooltipXOffset: 10,
                // String - Template string for single tooltips
                tooltipTemplate: "<%if (label){%><%=label%>: <%}%><%= value %>",
                // String - Template string for single tooltips
                multiTooltipTemplate: "<%= value %>",
                // Function - Will fire on animation progression.
                onAnimationProgress: function () {
                },
                // Function - Will fire on animation completion.
                onAnimationComplete: function () {
                },
                shouldDraw: function() {
                    var sum = 0;
                    for(i=i; i<$scope.myChartData.length-1; i++) {
                        sum += $scope.myChartData[i].value;
                    }
                    return sum !== 0;
                }
            };
            $scope.mapData = {
                initialized: false,
                view_initialized: false,
                route: $scope.route,
                collisions: [],
                offroad: [],
                speeding: [],
                tailgating: [],
                metadata: {
                    colors: {
                        route: "#00b9f1",
                        collision_route: "#f53721",
                        offroad_route: "#ffd800",
                        speeding_route: "#01ecb2",
                        none: "#aab4bd"
                    },
                    icons: {
                        start: new google.maps.MarkerImage(img_dir + "icon-source.png",
                            new google.maps.Size(21, 24),
                            new google.maps.Point(0, 0),
                            new google.maps.Point(10.5, 12),
                            new google.maps.Size(21, 24)),
                        end: new google.maps.MarkerImage(img_dir + "icon-destination.png",
                            new google.maps.Size(50, 10050),
                            new google.maps.Point(0, 0),
                            new google.maps.Point(25, 25),
                            new google.maps.Size(50, 50))
                    },
                    collision: {
                        icons: {
                            alert: new google.maps.MarkerImage(img_dir + "icon-collision.png",
                                new google.maps.Size(24, 40),
                                new google.maps.Point(0, 0),
                                new google.maps.Point(12, 44),
                                new google.maps.Size(24, 40))
                        }
                    },
                    offroad: {
                        icons: {
                            alert_left: new google.maps.MarkerImage(img_dir + "icon-offroad-left.png",
                                new google.maps.Size(28, 39),
                                new google.maps.Point(0, 0),
                                new google.maps.Point(13.25, 39),
                                new google.maps.Size(28, 39)),
                            alert_right: new google.maps.MarkerImage(img_dir + "icon-offroad-right.png",
                                new google.maps.Size(28, 39),
                                new google.maps.Point(0, 0),
                                new google.maps.Point(13.25, 39),
                                new google.maps.Size(28, 39))
                        }
                    },
                    speeding: {
                        icons: {
                            alert_happy: new google.maps.MarkerImage(img_dir + "icon-speeding-happy.png",
                                new google.maps.Size(24, 40),
                                new google.maps.Point(0, 0),
                                new google.maps.Point(12, 40),
                                new google.maps.Size(24, 40)),
                            alert_sad: new google.maps.MarkerImage(img_dir + "icon-speeding-sad.png",
                                new google.maps.Size(24, 40),
                                new google.maps.Point(0, 0),
                                new google.maps.Point(12, 40),
                                new google.maps.Size(24, 40))
                        }
                    },
                    tailgating: {
                        icons: {
                            //alert: new google.maps.MarkerImage(img_dir+"icon-tailgating.gif",
                            //    new google.maps.Size(90, 90),
                            //    new google.maps.Point(0, 0),
                            //    new google.maps.Point(22.5, 30),
                            //    new google.maps.Size(90, 90))
                            alert: {
                                path: google.maps.SymbolPath.CIRCLE,
                                scale: 0
                            }
                        }
                    }
                },
                activeMap: []
            };
            $scope.dimensions = _Utils.getWindowDimensions();

            $scope.startRunTime = "";
            $scope.totalDistance = 0;
            $scope.totalDistanceMetric = "km";
            $scope.totalTime = 0;

            $scope.totalCollisionAlerts = 0;
            $scope.maxCollisionAlerts = 40;
            $scope.totalSpeedingAlerts = 0;
            $scope.maxSpeedingAlerts = 15;
            $scope.totalTailgatingAlerts = 0;
            $scope.maxTailgatingAlerts = 25;
            $scope.totalOffroadAlerts = 0;
            $scope.maxOffroadAlerts = 20;

            // Initialize the current run
            $scope.drive = $scope._RunInfo;
            $scope.startRunTime = _Utils.getReadableTimestamp($scope.drive.StartTime);
            $scope.startRunTimeMs = $scope.drive.StartTime;
            $scope.totalTime = _Utils.getDuration($scope.drive.StartTime, $scope.drive.EndTime);
            $scope.totalTimeMetric = _Utils.getDurationMetric($scope.drive.StartTime, $scope.drive.EndTime);
            $scope.totalDistance = _Utils.getDistance($scope.drive.TotalDistance);
            $scope.totalDistanceMetric = _Utils.getDistanceMetric($scope.drive.TotalDistance, $scope.Settings.speed_units);

            // Initialize the route
            $scope.route = $scope._RouteInfo;
            $scope.mapData.route = $scope._RouteInfo;

            // Initialize the alerts
            var tailgatingAlertsCount = 0;
            if ($scope._Alerts.tailgating.length > 0) {
                var curr;
                var next;
                for(i=0;i<$scope._Alerts.tailgating.length-1;i++) {
                    curr = $scope._Alerts.tailgating[i];
                    next = $scope._Alerts.tailgating[i+1];

                    if (curr.state != 1 && next.state == 1) {
                        tailgatingAlertsCount++;
                    }
                }
            }
            var sum = $scope._Alerts.collision.length + $scope._Alerts.speeding.length + $scope._Alerts.offroad.length + tailgatingAlertsCount;
            $scope.totalCollisionAlerts = $scope._Alerts.collision.length;
            $scope.myChartData[0].value = (100 * ($scope._Alerts.collision.length / sum));
            $scope.myChartData[0].color = "#f53721";
            //$scope.myChartData[4].value = $scope.myChartData[4].value - $scope.myChartData[0].value;
            $scope.totalTailgatingAlerts = tailgatingAlertsCount;
            $scope.myChartData[1].value = (100 * (tailgatingAlertsCount / sum));
            $scope.myChartData[1].color = "#00b9f 1";
            //$scope.myChartData[4].value = $scope.myChartData[4].value - $scope.myChartData[1].value;
            $scope.totalOffroadAlerts = $scope._Alerts.offroad.length;
            $scope.myChartData[2].value = (100 * ($scope._Alerts.offroad.length / sum));
            $scope.myChartData[2].color = "#ffd800";
            //$scope.myChartData[4].value = $scope.myChartData[4].value - $scope.myChartData[2].value;
            $scope.totalSpeedingAlerts = $scope._Alerts.speeding.length;
            $scope.myChartData[3].value = (100 * ($scope._Alerts.speeding.length / sum));
            $scope.myChartData[3].color = "#01ecb2";
            //$scope.myChartData[4].value = $scope.myChartData[4].value - $scope.myChartData[3].value;
            // Initialize the map data
            $scope.mapData.collisions = $scope._Alerts.collision;
            $scope.mapData.offroad = $scope._Alerts.offroad;
            $scope.mapData.tailgating = $scope._Alerts.tailgating;
            $scope.mapData.speeding = $scope._Alerts.speeding;
            // This is the first view, we'll initialize the map in here.
            initializeMap();
        };

        $scope.changeState = function() {
            $scope.mapData.view_initialized = false;
        };

        $scope.$on('$viewContentLoaded', function (event) {
            if (!$scope.mapData.initialized || $scope.mapData.view_initialized) return;

            $scope.mapData.view_initialized = true;

            updateMap();
        });

        $scope.deleteRun = function (runId) {
            var confirmPopup = $ionicPopup.confirm({
                title: "Are you sure?",
                template: "The drive and it's data will be deleted for ever.",
                cancelText: 'No',
                cancelType: 'button-balanced',
                okText: 'Yes',
                okType: 'button-default',
            });
            confirmPopup.then(function(res) {
                if (res) { // delete the drive
                    RunsService.delete(runId)
                        .then(function(data) {
                            RunsService.count(function (count) {
                                if (count > 0) {
                                    if (_IsPro) {
                                        $ionicHistory.goBack(1);
                                    } else {
                                        $state.go('home');
                                    }
                                }
                                else {
                                    $state.go('home');
                                }
                            });
                        });
                }
            });
        };
        $scope.registerCollision = function (m) {
            // google.maps.event.addListener(m, 'click', function () {
            //     $scope.$apply(function () {
            //         $scope.showCollision(m.metadata);
            //     });
            // });
        };
        $scope.registerSpeeding = function (m) {
            // google.maps.event.addListener(m, 'click', function () {
            //     $scope.$apply(function () {
            //         $scope.showSpeeding(m.metadata);
            //     });
            // });
        };
        $scope.showCollision = function (collision) {
            if (collision.Snapshot) { // Lets show the collision card
                $scope.isShowCollision = true;
                $scope.collisionTime = moment($scope.startRunTimeMs).add(collision.Timestamp, "milliseconds").format("DD/MM/YYYY HH:mm");
                $scope.collisionPicture = collision.Snapshot;
            }
        };
        $scope.showSpeeding = function (speeding) {
            var speed_actual = speeding.Speed;
            var speed_sign = parseInt(speeding.SpeedSign
                .replace("USSpeed", "")
                .replace("Speed", ""));
            if (speed_sign >= speed_actual) {
                $scope.speedingType = "happy";
            } else {
                $scope.speedingType = "sad";
            }
            $scope.isShowSpeeding = true;
        };
        $scope.hideAlert = function () {
            $scope.isShowCollision = false;
            $scope.isShowSpeeding = false;
        };
        $scope.getStartRunTime = function () {
            return $scope.drive.StartTime;
        };
        $scope.getEndRunTime = function () {
            return $scope.drive.EndTime;
        };
        $scope.getTotalTime = function () {
            var startTime = moment($scope.drive.StartTime, "DD/MM/YYYY HH:mm:ss");
            var endTime = moment($scope.drive.EndTime, "DD/MM/YYYY HH:mm:ss");
            var time = (endTime.diff(startTime, "minutes"));
            return moment.duration(time, "minutes").asHours().toFixed(2);
        };
    }])

    .controller('SettingsCtrl', ["$scope", "$state", "_Utils", "_Settings", "$ionicLoading", "$ionicPopup", "RunsService", "UsersService", "_IsUserLoggedIn", "_SettingsData", function ($scope, $state, _Utils, _Settings, $ionicLoading, $ionicPopup, RunsService, UsersService, _IsUserLoggedIn, _SettingsData) {

        $scope._IsUserLoggedIn = _IsUserLoggedIn;

        var KPH = [50,60,70,80,90,100,110,120,130,140];
        var KPH_STEP = 10;
        var MPH = [30,35,40,45,50,55,60,65,70,75,80,85,90];
        var MPH_STEP = 5;
        var KPH_TO_MPH = [0,2,3,4,5,6,8,9,10,12];
        var MPH_TO_KPH = [0,1,1,2,3,4,5,5,6,7,8,8,9];

        setTimeout(function() {
            $scope.$apply(function() {
                $scope.VM = {
                    should_reset_settings: _SettingsData[0].promise.$$state.value,

                    debug_mode: _SettingsData[1].promise.$$state.value,
                    show_debug_info: _SettingsData[2].promise.$$state.value,
                    enable_sounds: _SettingsData[3].promise.$$state.value,
                    is_first_app_run: _SettingsData[4].promise.$$state.value,

                    speed_units: _SettingsData[5].promise.$$state.value,
                    marker_information: _SettingsData[6].promise.$$state.value,
                    list_navigation_app: _SettingsData[7].promise.$$state.value,
                    list_navigation_app_values: _SettingsData[8].promise.$$state.value,
                    selected_navigation_app_name: _SettingsData[9].promise.$$state.value,
                    selected_navigation_pkg_name: _SettingsData[10].promise.$$state.value,
                    parking_notification: _SettingsData[11].promise.$$state.value,

                    cloud_backup: _SettingsData[12].promise.$$state.value,

                    collision_alert: _SettingsData[13].promise.$$state.value,
                    off_road_warning: _SettingsData[14].promise.$$state.value,
                    tailgating_alert: _SettingsData[15].promise.$$state.value,
                    speeding_alert: _SettingsData[16].promise.$$state.value,

                    speed_sign_alert: _SettingsData[17].promise.$$state.value,
                    show_speed_sign: _SettingsData[18].promise.$$state.value,
                    show_speed_sign_values: _SettingsData[19].promise.$$state.value,
                    selected_show_speed_sign: _SettingsData[20].promise.$$state.value,
                    selected_tsr_time_value: _SettingsData[21].promise.$$state.value,
                    limit_max_speed: _SettingsData[22].promise.$$state.value,
                    max_speed_value: _SettingsData[23].promise.$$state.value,
                    speed_limit_range_top: _SettingsData[24].promise.$$state.value,
                    speed_limit_range_bottom: _SettingsData[25].promise.$$state.value,
                    selected_tailgating_alert_time_value: _SettingsData[26].promise.$$state.value,
                    selected_tailgating_alert_time: _SettingsData[27].promise.$$state.value,
                    show_hmw_values: _SettingsData[28].promise.$$state.value,
                    tailgating_alert_time: _SettingsData[29].promise.$$state.value,

                    limit_video_length: _SettingsData[30].promise.$$state.value,
                    video_max_length_value: _SettingsData[31].promise.$$state.value,
                    video_max_length: _SettingsData[32].promise.$$state.value,
                    video_max_length_names: _SettingsData[33].promise.$$state.value,
                    video_max_length_values: _SettingsData[34].promise.$$state.value,

                    prodashboard: _SettingsData[35].promise.$$state.value,
                };
            });

            if ($scope.VM.speed_units) {
                $scope.VM.max_speed_value_normalized = ($scope.VM.max_speed_value - KPH[0]) / KPH_STEP;
            } else {
                $scope.VM.max_speed_value_normalized = ($scope.VM.max_speed_value - MPH[0]) / MPH_STEP;
            }

        }, 500);

        $scope.reset_settings = function () {
            $scope.updateKey('should_reset_settings', true, "boolean");

            var timer = setInterval(function () {
                _Settings.getValue("should_reset_settings", "boolean",
                    function (result) {
                        if (!result) {
                            clearInterval(timer);
                            $scope.load(true);

                            $ionicLoading.show({
                                template: "" +
                                "<div>" +
                                "<div>" +
                                "Your wish is my command..." +
                                "</div>" +
                                "<div class='settings-spinner'>" +
                                "<ion-spinner class='spinner-light'></ion-spinner>" +
                                "</div>" +
                                "</div>"
                            });

                            setTimeout(function () {
                                $ionicLoading.hide();
                            }, 3000);
                        }
                    },
                    function (error) {
                        return true;
                    });
            }, 500);
        };

        $scope.show_dev_trick = function() {
            $scope.show_dev_options++;
        };

        $scope.init = function () {
            //$scope.load(false);
            $scope.show_dev_options = 0;
        };

        $scope.load = function (isReset) {

            $scope.VM = {};

            if (!_Settings.initialized())
                return;

            _Settings.load();

            // Common Settings
            _Settings.getValue("should_reset_settings", "boolean",
                function (result) {
                    if (!isReset) {
                        $scope.VM.should_reset_settings = result;
                    } else {
                        setTimeout(function () {
                            $scope.$apply(function () {
                                $scope.VM.should_reset_settings = result;
                            });
                        }, 1000);
                    }
                },
                function (error) {
                    return true;
                });

            // Dev Settings
            _Settings.getValue("debug_mode", "boolean",
                function (result) {
                    if (!isReset) {
                        $scope.VM.debug_mode = result;
                    } else {
                        setTimeout(function () {
                            $scope.$apply(function () {
                                $scope.VM.debug_mode = result;
                            });
                        }, 1000);
                    }
                },
                function (error) {
                    return true;
                });
            _Settings.getValue("show_debug_info", "boolean",
                function (result) {
                    if (!isReset) {
                        $scope.VM.show_debug_info = result;
                    } else {
                        setTimeout(function () {
                            $scope.$apply(function () {
                                $scope.VM.show_debug_info = result;
                            });
                        }, 1000);
                    }
                },
                function (error) {
                    return true;
                });
            _Settings.getValue("enable_sounds", "boolean",
                function (result) {
                    if (!isReset) {
                        $scope.VM.enable_sounds = result;
                    } else {
                        setTimeout(function () {
                            $scope.$apply(function () {
                                $scope.VM.enable_sounds = result;
                            });
                        }, 1000);
                    }
                },
                function (error) {
                    return true;
                });
            _Settings.getValue("is_first_app_run", "boolean",
                function (result) {
                    if (!isReset) {
                        $scope.VM.is_first_app_run = result;
                    } else {
                        setTimeout(function () {
                            $scope.$apply(function () {
                                $scope.VM.is_first_app_run = result;
                            });
                        }, 1000);
                    }
                },
                function (error) {
                    return true;
                });

            // Basic Settings
            _Settings.getValue("speed_units", "boolean",
                function (result) {
                    if (!isReset) {
                        $scope.VM.speed_units = result;
                    } else {
                        setTimeout(function () {
                            $scope.$apply(function () {
                                $scope.VM.speed_units = result;
                            });
                        }, 1000);
                    }
                },
                function (error) {
                    return true;
                });
            _Settings.getValue("marker_information", "boolean",
                function (result) {
                    if (!isReset) {
                        $scope.VM.marker_information = result;
                    } else {
                        setTimeout(function () {
                            $scope.$apply(function () {
                                $scope.VM.marker_information = result;
                            });
                        }, 1000);
                    }
                },
                function (error) {
                    return false;
                });
            _Settings.getValue("list_navigation_app", "array",
                function (result) {
                    $scope.VM.list_navigation_app = result;
                },
                function (error) {
                    return [];
                });
            _Settings.getValue("list_navigation_app_values", "array",
                function (result) {
                    $scope.VM.list_navigation_app_values = result;
                },
                function (error) {
                    return [];
                });
            _Settings.getValue("selected_navigation_app_name", "string",
                function (result) {
                    $scope.VM.selected_navigation_app_name = result;
                },
                function (error) {
                    return "";
                });
            _Settings.getValue("selected_navigation_pkg_name", "string",
                function (result) {
                    $scope.VM.selected_navigation_pkg_name = result;
                },
                function (error) {
                    return "";
                });
            _Settings.getValue("parking_notification", "boolean",
                function (result) {
                    if (!isReset) {
                        $scope.VM.parking_notification = result;
                    } else {
                        setTimeout(function () {
                            $scope.$apply(function () {
                                $scope.VM.parking_notification = result;
                            });
                        }, 1000);
                    }
                },
                function (error) {
                    return false;
                });

            //// Backup Settings
            _Settings.getValue("cloud_backup", "boolean",
                function (result) {
                    if (!isReset) {
                        $scope.VM.cloud_backup = result;
                    } else {
                        setTimeout(function () {
                            $scope.$apply(function () {
                                $scope.VM.cloud_backup = result;
                            });
                        }, 1000);
                    }
                },
                function (error) {
                    return false;
                });

            //// Alerts Settings
            _Settings.getValue("collision_alert", "boolean",
                function (result) {
                    if (!isReset) {
                        $scope.VM.collision_alert = result;
                    } else {
                        setTimeout(function () {
                            $scope.$apply(function () {
                                $scope.VM.collision_alert = result;
                            });
                        }, 1000);
                    }
                },
                function (error) {
                    return false;
                });
            _Settings.getValue("off_road_warning", "boolean",
                function (result) {
                    if (!isReset) {
                        $scope.VM.off_road_warning = result;
                    } else {
                        setTimeout(function () {
                            $scope.$apply(function () {
                                $scope.VM.off_road_warning = result;
                            });
                        }, 1000);
                    }
                },
                function (error) {
                    return false;
                });
            _Settings.getValue("tailgating_alert", "boolean",
                function (result) {
                    if (!isReset) {
                        $scope.VM.tailgating_alert = result;
                    } else {
                        setTimeout(function () {
                            $scope.$apply(function () {
                                $scope.VM.tailgating_alert = result;
                            });
                        }, 1000);
                    }
                },
                function (error) {
                    return false;
                });
            _Settings.getValue("speeding_alert", "boolean",
                function (result) {
                    if (!isReset) {
                        $scope.VM.speeding_alert = result;
                    } else {
                        setTimeout(function () {
                            $scope.$apply(function () {
                                $scope.VM.speeding_alert = result;
                            });
                        }, 1000);
                    }
                },
                function (error) {
                    return false;
                });

            //// Advanced Settings
            _Settings.getValue("speed_sign_alert", "boolean",
                function (result) {
                    if (!isReset) {
                        $scope.VM.speed_sign_alert = result;
                    } else {
                        setTimeout(function () {
                            $scope.$apply(function () {
                                $scope.VM.speed_sign_alert = result;
                            });
                        }, 1000);
                    }
                },
                function (error) {
                    return false;
                });
            _Settings.getValue("show_speed_sign", "array",
                function (result) {
                    $scope.VM.show_speed_sign = result;
                },
                function (error) {
                    return [];
                });
            _Settings.getValue("show_speed_sign_values", "array",
                function (result) {
                    $scope.VM.show_speed_sign_values = result;
                },
                function (error) {
                    return [];
                });
            _Settings.getValue("selected_show_speed_sign", "string",
                function (result) {
                    $scope.VM.selected_show_speed_sign = result;
                },
                function (error) {
                    return false;
                });
            _Settings.getValue("selected_tsr_time_value", "long",
                function (result) {
                    $scope.VM.selected_tsr_time_value = result;
                },
                function (error) {
                    return false;
                });
            _Settings.getValue("limit_max_speed", "boolean",
                function (result) {
                    if (!isReset) {
                        $scope.VM.limit_max_speed = result;
                    } else {
                        setTimeout(function () {
                            $scope.$apply(function () {
                                $scope.VM.limit_max_speed = result;
                            });
                        }, 1000);
                    }
                },
                function (error) {
                    return false;
                });
            _Settings.getValue("max_speed_value", "int",
                function (result) {
                    if (!isReset) {
                        $scope.VM.max_speed_value = result;
                    } else {
                        setTimeout(function () {
                            $scope.$apply(function () {
                                $scope.VM.max_speed_value = result;

                                if ($scope.VM.speed_units) {
                                    $scope.VM.max_speed_value_normalized = ($scope.VM.max_speed_value - KPH[0]) / KPH_STEP;
                                } else {
                                    $scope.VM.max_speed_value_normalized = ($scope.VM.max_speed_value - MPH[0]) / MPH_STEP;
                                }
                            });
                        }, 1000);
                    }
                },
                function (error) {
                    return 65;
                });
            _Settings.getValue("speed_limit_range_top", "int",
                function (result) {
                    if (!isReset) {
                        $scope.VM.speed_limit_range_top = result;
                    } else {
                        setTimeout(function () {
                            $scope.$apply(function () {
                                $scope.VM.speed_limit_range_top = result;
                            });
                        }, 1000);
                    }
                },
                function (error) {
                    return 140;
                });
            _Settings.getValue("speed_limit_range_bottom", "int",
                function (result) {
                    if (!isReset) {
                        $scope.VM.speed_limit_range_bottom = result;
                    } else {
                        setTimeout(function () {
                            $scope.$apply(function () {
                                $scope.VM.speed_limit_range_bottom = result;
                            });
                        }, 1000);
                    }
                },
                function (error) {
                    return 50;
                });
            //_Settings.getValue("tailgating_alert_time", "int",
            //    function(result) {
            // $scope.tailgating_alert_time = result / 1000;
            // },
            //    function(error) {
            // return 1;
            // });
            _Settings.getValue("selected_tailgating_alert_time_value", "int",
                function (result) {
                    $scope.VM.selected_tailgating_alert_time_value = result / 1000;
                },
                function (error) {
                    return 0;
                });
            _Settings.getValue("selected_tailgating_alert_time", "string",
                function (result) {
                    $scope.VM.selected_tailgating_alert_time = result;
                },
                function (error) {
                    return "";
                });
            _Settings.getValue("show_hmw_values", "array",
                function (result) {
                    $scope.VM.show_hmw_values = result;
                },
                function (error) {
                    return [];
                });
            _Settings.getValue("tailgating_alert_time", "array",
                function (result) {
                    $scope.VM.tailgating_alert_time = result;
                },
                function (error) {
                    return [];
                });

            _Settings.getValue("limit_video_length", "boolean",
                function (result) {
                    if (!isReset) {
                        $scope.VM.limit_video_length = result;
                    } else {
                        setTimeout(function () {
                            $scope.$apply(function () {
                                $scope.VM.limit_video_length = result;
                            });
                        }, 1000);
                    }
                },
                function (error) {
                    return false;
                });
            //_Settings.getValue("video_max_length", "int",
            //    function(result) {
            // $scope.video_max_length = result;
            // },
            //    function(error) {
            // return 0;
            // });
            _Settings.getValue("video_max_length_value", "int",
                function (result) {
                    $scope.VM.video_max_length_value = result;
                },
                function (error) {
                    return [];
                });
            _Settings.getValue("video_max_length", "string",
                function (result) {
                    $scope.VM.video_max_length = result;
                },
                function (error) {
                    return [];
                });
            _Settings.getValue("video_max_length_names", "array",
                function (result) {
                    $scope.VM.video_max_length_names = result;
                },
                function (error) {
                    return false;
                });
            _Settings.getValue("video_max_length_values", "array",
                function (result) {
                    $scope.VM.video_max_length_values = result;
                },
                function (error) {
                    return false;
                });

            // New
            _Settings.getValue("com.ionroad.prodashboard", "boolean",
                function (result) {
                    if (!isReset) {
                        $scope.VM.prodashboard = result;
                    } else {
                        setTimeout(function () {
                            $scope.$apply(function () {
                                $scope.VM.prodashboard = result;
                            });
                        }, 1000);
                    }
                },
                function (error) {
                    return true;
                });
        };

        $scope.updateKey = function (key, value, type) {
            _Settings.putValue(key, value, type,
                function (result) {
                    console.log("Successfully written to the settings file", key, value, result);
                },
                function (error) {
                    console.log("Error writing to the settings file", key, value, error);
                });
        };

        $scope.updateDebugMode = function () {
            $scope.updateKey('debug_mode', $scope.VM.debug_mode, "boolean");
            $scope.updateKey('test_mode', $scope.VM.debug_mode, "boolean");
        };

        $scope.updateSpeedUnit = function () {
            $scope.updateKey('speed_units', $scope.VM.speed_units, "boolean");

            $ionicLoading.show({
                template: "" +
                "<div>" +
                "<div>" +
                "Momento..." +
                "</div>" +
                "<div class='settings-spinner'>" +
                "<ion-spinner class='spinner-light'></ion-spinner>" +
                "</div>" +
                "</div>"
            });

            setTimeout(function () {
                var isReset = true;

                _Settings.getValue("max_speed_value", "int",
                    function (result) {
                        if (!isReset) {
                            $scope.VM.max_speed_value = result;
                        } else {
                            setTimeout(function () {
                                $scope.$apply(function () {
                                    $scope.VM.max_speed_value = result;

                                    if ($scope.VM.speed_units) {
                                        $scope.VM.max_speed_value_normalized = ($scope.VM.max_speed_value - KPH[0]) / KPH_STEP;
                                    } else {
                                        $scope.VM.max_speed_value_normalized = ($scope.VM.max_speed_value - MPH[0]) / MPH_STEP;
                                    }
                                });

                                $ionicLoading.hide();
                            }, 1000);
                        }
                    },
                    function (error) {
                        return 65;
                    });
                _Settings.getValue("speed_limit_range_top", "int",
                    function (result) {
                        if (!isReset) {
                            $scope.VM.speed_limit_range_top = result;
                        } else {
                            setTimeout(function () {
                                $scope.$apply(function () {
                                    $scope.VM.speed_limit_range_top = result;
                                });
                            }, 1000);
                        }
                    },
                    function (error) {
                        return 140;
                    });
                _Settings.getValue("speed_limit_range_bottom", "int",
                    function (result) {
                        if (!isReset) {
                            $scope.VM.speed_limit_range_bottom = result;
                        } else {
                            setTimeout(function () {
                                $scope.$apply(function () {
                                    $scope.VM.speed_limit_range_bottom = result;
                                });
                            }, 1000);
                        }
                    },
                    function (error) {
                        return 50;
                    });
            }, 500);
        };

        $scope.updateCloudBackup = function(value) {
            if (value) {
                if (!$scope._IsUserLoggedIn) {
                    // User is not logged in yet,
                    // Redirect to login screen, which in turn, will
                    // Turn on cloud backup
                    $state.go('login', {return_state:'settings'});
                }
            } else {
                if ($scope._IsUserLoggedIn) {
                    var confirmPopup = $ionicPopup.confirm({
                        title: "Are you sure?",
                        template: "You will not have backup for your drives data anymore",
                        cancelText: 'No',
                        cancelType: 'button-balanced',
                        okText: 'Yes',
                        okType: 'button-default',
                    });
                    confirmPopup.then(function(res) {
                        if (res) { // delete the drive
                            $ionicLoading.show({
                                template: "" +
                                "<div>" +
                                "<div>" +
                                "Your wish is my command..." +
                                "</div>" +
                                "<div class='settings-spinner'>" +
                                "<ion-spinner class='spinner-light'></ion-spinner>" +
                                "</div>" +
                                "</div>"
                            });

                            // Turn off cloud backup
                            $scope.updateKey('cloud_backup', value, "boolean");

                            // Set the user as logged-out
                            $scope._IsUserLoggedIn = false;

                            // Log the user out
                            UsersService.setUserInfo(null, null, null, function () {
                                $ionicLoading.hide();
                            });
                        }
                        else {
                            $scope.VM.cloud_backup = true;
                            $scope.$apply();
                        }
                    });
                }
            }
        };

        $scope.updateMaxSpeedLimit = function (limit) {
            if ($scope.VM.speed_units) {
                $scope.VM.max_speed_value = KPH[0] + (limit * KPH_STEP);
            } else {
                $scope.VM.max_speed_value = MPH[0] + (limit * MPH_STEP);
            }

            $scope.updateKey('max_speed_value', $scope.VM.max_speed_value, "int");
        };

        $scope.updateTailgatingAlertTime = function () {
            $scope.updateKey('tailgating_alert_time', $scope.VM.tailgating_alert_time * 1000, "int");
        };

        $scope.updateSelectedNavApp = function (app) {
            var i = _.indexOf($scope.VM.list_navigation_app, app);
            $scope.updateKey('selected_navigation_app_name', app, "string");
            $scope.updateKey('selected_navigation_pkg_name', $scope.VM.list_navigation_app_values[i], "string");
        };

        $scope.updateShowSpeedSign = function (option) {
            var i = _.indexOf($scope.VM.show_speed_sign, option);
            $scope.updateKey('selected_show_speed_sign', option, "string");
            $scope.updateKey('selected_tsr_time_value', Number($scope.VM.show_speed_sign_values[i]), "long");
        };

        $scope.updateTailgatingAlertTime = function (option) {
            var i = _.indexOf($scope.VM.tailgating_alert_time, option);
            $scope.updateKey('selected_tailgating_alert_time', option, "string");
            $scope.updateKey('selected_tailgating_alert_time_value', parseInt($scope.VM.show_hmw_values[i]), "int");
        };

        $scope.updateVideoMaxLength = function (option) {
            var i = _.indexOf($scope.VM.video_max_length_names, option);
            $scope.updateKey('video_max_length', option, "string");
            $scope.updateKey('video_max_length_value', parseInt($scope.VM.video_max_length_values[i]), "int");
        };

        $scope.clearLocalData = function() {
            var confirmPopup = $ionicPopup.confirm({
                title: "Are you sure?",
                template: "All drives and their data will be deleted for ever.",
                cancelText: 'No',
                cancelType: 'button-balanced',
                okText: 'Yes',
                okType: 'button-default',
            });
            confirmPopup.then(function(res) {
                if (res) { // delete the drive
                    $ionicLoading.show({
                        template: "" +
                        "<div>" +
                        "<div>" +
                        "Your wish is my command..." +
                        "</div>" +
                        "<div class='settings-spinner'>" +
                        "<ion-spinner class='spinner-light'></ion-spinner>" +
                        "</div>" +
                        "</div>"
                    });

                    RunsService.clear_local_data()
                        .then(function(data) {
                            $ionicLoading.hide();
                            console.log("Successfully cleared all local data");
                        });
                }
            });
        };
    }])

    .controller('ParkingCtrl', ["$scope", "$http", "$state", "$stateParams", "$cordovaGeolocation", "$cordovaDatePicker", "$cordovaCamera", "$ionicHistory", "$ionicPopup", "ParkingService", "RunsService", "RoutesService", "_Utils", "_Settings", "_ReminderInfo", "_ParkingInfo", "_ParkingPhoto", function($scope, $http, $state, $stateParams, $cordovaGeolocation, $cordovaDatePicker, $cordovaCamera, $ionicHistory, $ionicPopup, ParkingService, RunsService, RoutesService, _Utils, _Settings, _ReminderInfo, _ParkingInfo, _ParkingPhoto) {

        var LOCAL_NOTIFICATION_ID = 4804434;

        $scope._ParkingInfo = _ParkingInfo;
        $scope._ParkingPhoto = _ParkingPhoto;
        $scope._ReminderInfo = _ReminderInfo;

        $scope.center_on_me = function() {
            $scope.mapData.map.setCenter(new google.maps.LatLng(
                $scope.mapData.location.user.lat,
                $scope.mapData.location.user.long));
            $scope.mapData.map.setZoom(16);

            $scope.mapData.user_tracking = true;
        };

        $scope.center_on_car = function() {
            $scope.mapData.map.setCenter(new google.maps.LatLng(
                $scope.mapData.location.parking.lat,
                $scope.mapData.location.parking.long));
            $scope.mapData.map.setZoom(16);

            $scope.mapData.user_tracking = false;
        };

        var rad = function(x) {
          return x * Math.PI / 180;
        };

        var getDistance = function(p1, p2) {
            var R = 6378137; // Earth's mean radius in meter
            var dLat = rad(p2.lat - p1.lat);
            var dLong = rad(p2.long - p1.long);
            var a = Math.sin(dLat / 2) * Math.sin(dLat / 2) +
                Math.cos(rad(p1.lat)) * Math.cos(rad(p2.lat)) *
                Math.sin(dLong / 2) * Math.sin(dLong / 2);
            var c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
            var d = R * c;
            return d; // returns the distance in meter
        };

        // function CarLocationControl(controlDiv, map) {
        //     // Set CSS for the control border
        //     var controlUI = document.createElement('img');
        //     controlUI.src = "img/map/icon-car.png";
        //     controlUI.width = 32;
        //     controlUI.height = 32;
        //     controlUI.style.marginLeft = "10px";
        //     controlUI.style.marginTop = "-5px";
        //     controlUI.style.display = "block";
        //     controlUI.style.opacity = 0.85;
        //     controlDiv.appendChild(controlUI);
        //
        //     // Setup the click event listeners: simply set the map to
        //     // Chicago
        //     google.maps.event.addDomListener(controlUI, 'click', function () {
        //         map.setCenter(new google.maps.LatLng(
        //             $scope.mapData.location.parking.lat,
        //             $scope.mapData.location.parking.long));
        //         map.setZoom(16);
        //     });
        // }
        //
        // function UserLocationControl(controlDiv, map) {
        //     // Set CSS for the control border
        //     var controlUI = document.createElement('img');
        //     controlUI.src = "img/map/icon-me.png";
        //     controlUI.width = 32;
        //     controlUI.height = 32;
        //     controlUI.style.marginLeft = "10px";
        //     controlUI.style.marginTop = "5px";
        //     controlUI.style.display = "block";
        //     controlUI.style.opacity = 0.85;
        //     controlDiv.appendChild(controlUI);
        //
        //     // Setup the click event listeners: simply set the map to
        //     // Chicago
        //     google.maps.event.addDomListener(controlUI, 'click', function () {
        //         map.setCenter(new google.maps.LatLng(
        //             $scope.mapData.location.user.lat,
        //             $scope.mapData.location.user.long));
        //         map.setZoom(16);
        //     });
        // }

        function setMarker(map, icon, coor) {
            return new google.maps.Marker({
                position: coor,
                map: map,
                icon: icon,
                optimized: false
            });
        }

        function setMarkerWithLabel(map, icon, coor, labelClass) {
            return new MarkerWithLabel({
                position: coor,
                map: map,
                icon: icon,
                labelAnchor: new google.maps.Point(10, 10),
                labelClass: labelClass
            });
        }

        function initializeMap() {
            if (typeof(google)==="undefined") {
                return;
            }

            var myLatlng;
            if ($state.includes('parking')) {
                myLatlng = new google.maps.LatLng($scope.mapData.location.parking.lat, $scope.mapData.location.parking.long);
            }
            else if ($state.includes('parking_location_change')) {
                myLatlng = new google.maps.LatLng($scope.mapData.location.new_parking.lat, $scope.mapData.location.new_parking.long);
            }
            else {
                // We shouldn't reach here but just in case...
                myLatlng = new google.maps.LatLng($scope.mapData.location.parking.lat, $scope.mapData.location.parking.long);
            }

            var mapOptions = {
                center: myLatlng,
                zoom: 16,
                mapTypeId: google.maps.MapTypeId.ROADMAP,
                panControl: false,
                zoomControl: false,
                mapTypeControl: false,
                scaleControl: false,
                streetViewControl: false,
                overviewMapControl: false
            };
            var map = new google.maps.Map(document.getElementById($scope.mapData.options.mapElement),
                mapOptions);

            $scope.mapData.map = map;

            $scope.mapData.markers.parking = setMarker(map,
                new google.maps.MarkerImage(img_dir + "icon-park.png",
                    new google.maps.Size(27, 38),
                    new google.maps.Point(0, 0),
                    new google.maps.Point(13, 25),
                    new google.maps.Size(27, 38)),
                new google.maps.LatLng($scope.mapData.location.parking.lat, $scope.mapData.location.parking.long));
        }

        $scope.trackUserLocation = function() {
            // var posOptions = {timeout: 10000, enableHighAccuracy: false};
            // $cordovaGeolocation
            //     .getCurrentPosition(posOptions)
            //     .then(function (position) {
            //         if (($scope.mapData.location.user.latitude != position.coords.latitude) || ($scope.mapData.location.user.longitude != position.coords.longitude)) {
            //             $scope.mapData.location.user = {
            //                 lat: position.coords.latitude,
            //                 long: position.coords.longitude
            //             };
            //
            //             // Lets first clear the position of the user from the map
            //             if ($scope.mapData.markers.user)
            //                 $scope.mapData.markers.user.setMap(null);
            //
            //             if (!$scope.mapData.location.new_parking.init) {
            //                 $scope.mapData.location.new_parking.lat = position.coords.latitude;
            //                 $scope.mapData.location.new_parking.long = position.coords.longitude;
            //                 $scope.mapData.location.new_parking.init = true;
            //
            //                 // TODO: Need to figure out why this is an issue
            //                 // $scope.mapData.map.setCenter(new google.maps.LatLng(position.coords.latitude, position.coords.longitude);
            //             }
            //
            //             // Lets re-draw the current position of the user on the map
            //             $scope.mapData.markers.user = setMarkerWithLabel($scope.mapData.map,
            //                 {path: google.maps.SymbolPath.CIRCLE, scale: 0},
            //                 new google.maps.LatLng(position.coords.latitude, position.coords.longitude),
            //                 "gps");
            //
            //             var distance = getDistance($scope.mapData.location.user, $scope.mapData.location.parking);
            //             if (distance < 1000) {
            //                 var meters = distance.toFixed(0);
            //                 $scope.distance = meters + " m";
            //             }
            //             else {
            //                 distance = distance / 1000;
            //                 var kms = distance.toFixed(1);
            //                 $scope.distance = kms + " km";
            //             }
            //         }
            //     });

            // Keep watching the location of the user
            var watchOptions = {
                frequency: 5000,
                timeout: 10000,
                enableHighAccuracy: false // may cause errors if true
            };
            $scope.position_watcher = $cordovaGeolocation.watchPosition(watchOptions);
            $scope.position_watcher.then(
                null,
                function (err) {
                    console.log("Couldn't get the GPS coordinates", err);
                },
                function (position) {
                    if (($scope.mapData.location.user.latitude != position.coords.latitude) || ($scope.mapData.location.user.longitude != position.coords.longitude)) {
                        $scope.mapData.location.user = {
                            lat: position.coords.latitude,
                            long: position.coords.longitude
                        };

                        // Lets first clear the position of the user from the map
                        // if ($scope.mapData.markers.user)
                        //     $scope.mapData.markers.user.setMap(null);

                        $scope.mapData.location.new_parking.lat = position.coords.latitude;
                        $scope.mapData.location.new_parking.long = position.coords.longitude;

                        // continuously center map, if tracking is enabled
                        if ($scope.mapData.user_tracking) {
                            $scope.mapData.map.setCenter(new google.maps.LatLng(position.coords.latitude, position.coords.longitude));
                        }

                        // Lets re-draw the current position of the user on the map
                        if (!$scope.mapData.markers.user) {
                            // At the first time, lets draw the marker
                            $scope.mapData.markers.user = setMarkerWithLabel($scope.mapData.map,
                                {path: google.maps.SymbolPath.CIRCLE, scale: 0},
                                new google.maps.LatLng(position.coords.latitude, position.coords.longitude),
                                "gps");
                        } else {
                            // Now, lets only change the position of the marker on the map
                            $scope.mapData.markers.user.setPosition(
                                new google.maps.LatLng(position.coords.latitude, position.coords.longitude));
                        }

                        var distance = getDistance($scope.mapData.location.user, $scope.mapData.location.parking);
                        if (distance < 1000) {
                            var meters = distance.toFixed(0);
                            $scope.distance = meters + " m";
                        }
                        else {
                            distance = distance / 1000;
                            var kms = distance.toFixed(1);
                            $scope.distance = kms + " km";
                        }
                    }
                });
        };

        $scope.init = function () {
            $scope.dimensions = _Utils.getWindowDimensions();

            // Initialize the VM.
            $scope.alarm = null;
            $scope.mapData = {
                map: null,
                options: {},
                user_tracking: false,
                location: {
                    parking: {
                        lat: 0,
                        long: 0
                    },
                    new_parking: {
                        lat: 0,
                        long: 0,
                        init: false
                    },
                    user: {
                        lat: 0,
                        long: 0,
                    }
                },
                markers: {
                    parking: null,
                    user: null
                },
                images: {
                    parking: null
                }
            };
            $scope.dimensions = _Utils.getWindowDimensions();

            if ($state.includes('parking')) {
                $scope.mapData.options.mapElement = "park-main-map";
            }
            else if ($state.includes('parking_location_change')) {
                $scope.mapData.options.mapElement = "park-location-map";
                $scope.mapData.user_tracking = true;
            }

            // Get parking info
            $scope.parking = $scope._ParkingInfo;
            $scope.parking.ReadableTime = _Utils.getReadableTimestamp($scope.parking.Timestamp);
            $scope.mapData.location.parking.lat = $scope.parking.Latitude;
            $scope.mapData.location.parking.long = $scope.parking.Longitude;
            if ($scope.mapData.options.mapElement) {
                initializeMap();
                console.log($scope.mapData);
            }

            // Track the users' location
            $scope.trackUserLocation();

            // Initially, center map on parking spot
            if ($scope.mapData.map) {
                // This is needed when changing the dimensions of the map dynamically
                document.getElementById("park-main-map").style.height = ($scope.dimensions.height-317)+"px";
                google.maps.event.trigger($scope.mapData.map, 'resize');

                $scope.mapData.map.setCenter(new google.maps.LatLng($scope.mapData.location.parking.lat, $scope.mapData.location.parking.long));
            }

            // Get reminder info
            if ($scope._ReminderInfo) {
                $scope.alarm = moment($scope._ReminderInfo).toDate();
            }

            // Get parking photo
            $scope.mapData.images.parking = $scope._ParkingPhoto;
        };

        $scope.$on('$destroy', function () {
            if ($scope.position_watcher) {
                $scope.position_watcher.clearWatch();
            }
        });

        $scope.takePicture = function () {
            var options = {
                quality: 100,
                destinationType: Camera.DestinationType.DATA_URL,
                sourceType: Camera.PictureSourceType.CAMERA,
                allowEdit: false,
                encodingType: Camera.EncodingType.JPEG,
                targetWidth: 640,
                targetHeight: 480,
                popoverOptions: CameraPopoverOptions,
                saveToPhotoAlbum: false,
                correctOrientation: true
            };

            if (!_Settings.initialized()) {
                return;
            }

            _Settings.load();

            // Lets make sure the user can't switch back to the main
            // application while using the camera
            _Settings.putValue("should_lock_orientation", true, "boolean",
                function (result) {
                    console.log("Succesfully written to the settings the first run of the app");
                }, function (error) {
                    console.log("Error writing to the settings that the app ran for the first time");
                });

            $cordovaCamera.getPicture(options).then(function (imageData) {
                // Now that the picture is taken, the user can switch
                // back to the main app
                _Settings.putValue("should_lock_orientation", false, "boolean",
                    function (result) {
                        console.log("Succesfully written to the settings the first run of the app");
                    }, function (error) {
                        console.log("Error writing to the settings that the app ran for the first time");
                    });

                $scope.mapData.images.new_parking = imageData;
            }, function (err) {
                // TODO: change to a placeholder?
            });
        };

        $scope.setPicture = function () {
            $scope.mapData.images.parking = $scope.mapData.images.new_parking;
            $scope.mapData.images.new_parking = null;

            ParkingService.setParkingPhoto($scope.mapData.images.parking, function () {
                // Nothing to do here.
                $state.go("parking");
            });
        };

        $scope.ignorePicture = function () {
            $scope.mapData.images.new_parking = null;
            $state.go('parking');
        };

        $scope.setLocation = function () {
            $scope.mapData.location.parking = $scope.mapData.location.new_parking;
            $scope.mapData.images.parking = null;

            $http.get("http://maps.googleapis.com/maps/api/geocode/json?latlng=" + $scope.mapData.location.parking.lat + "," + $scope.mapData.location.parking.long + "&sensor=true")
                .success(function(data, status, headers, config) {
                    if (data.results.length === 0) {
                        // Do nothing.
                    } else {
                        var o = data.results[0].address_components,
                            timestamp = new Date().getTime(),
                            lat = $scope.mapData.location.parking.lat,
                            long = $scope.mapData.location.parking.long,
                            country,
                            city,
                            address,
                            street_number;

                        _.forEach(o, function(n, key) {
                            if (_.include(n.types, "street_number")) {
                                street_number = n.long_name;
                            }
                            else if (_.include(n.types, "route")) {
                                address = n.long_name;
                            }
                            else if (_.include(n.types, "locality")) {
                                city = n.long_name;
                            }
                            else if (_.include(n.types, "country")) {
                                country = n.long_name;
                            }
                        });

                        ParkingService.setParkingLocation(timestamp, long, lat, country, city, address+" "+street_number, function () {
                            // Nothing to do here.
                            $state.go("parking");
                        });
                    }

                });
        };

        $scope.ignoreLocation = function () {
            $state.go('parking');
        };

        function registerAlarm(date) {
            var sound = device.platform == 'Android' ? 'file://sound.mp3' : 'file://beep.caf';

            cordova.plugins.notification.local.schedule({
                id: LOCAL_NOTIFICATION_ID,
                title: "iOnRoad",
                message: "Want to get back to your car? click here!",
                at: date,
                //sound: sound,
                icon: "http://domain.com/icon.png"
            });

            ParkingService.setAlarm(new Date(date).getTime(), function () {
                // Nothing to do here.
            });
        }

        function cancelAlarm() {
            // Cancel notification - notice that the 1 should equal the unique id of the notification
            cordova.plugins.notification.local.cancel(LOCAL_NOTIFICATION_ID, function () {
                // Notification was cancelled
            }, $scope);

            ParkingService.cancelAlarm(function () {
                // Nothing to do here.
            });
        }

        $scope.setAlarm = function () {
            var options = {
                date: new Date(),
                mode: 'time', // or 'date'
                minDate: new Date() - 10000,
                allowOldDates: true,
                allowFutureDates: false,
                doneButtonLabel: 'DONE',
                doneButtonColor: '#F2F3F4',
                cancelButtonLabel: 'CANCEL',
                cancelButtonColor: '#000000'
            };
            $cordovaDatePicker.show(options).then(function (date) {
                $scope.alarm = date;

                // Check for permission
                cordova.plugins.notification.local.hasPermission(function (granted) {
                    if (granted) {
                        registerAlarm(date);
                    } else {
                        // Ask for permission
                        cordova.plugins.notification.local.registerPermission(function (granted) {
                            if (granted) {
                                registerAlarm(date);
                            }
                        });
                    }
                });
            });
        };

        $scope.removeAlarm = function () {
            var confirmPopup = $ionicPopup.confirm({
                title: "Are you sure?",
                template: "The reminder will be deleted",
                cancelText: 'No',
                cancelType: 'button-balanced',
                okText: 'Yes',
                okType: 'button-default',
            });
            confirmPopup.then(function(res) {
                if (res) {
                    $scope.alarm = null;
                    cancelAlarm();
                }
            });
        };
    }]);
