(function (ionic) {
    var extend = angular.extend,
        forEach = angular.forEach,
        isDefined = angular.isDefined,
        isNumber = angular.isNumber,
        isString = angular.isString,
        jqLite = angular.element,
        noop = angular.noop;

    angular.module('iOnRoad.directives', [])

    /**
     * @ngdoc directive
     * @name iorToggle
     * @module ionic
     * @codepen tfAzj
     * @restrict E
     *
     * @description
     * A toggle is an animated switch which binds a given model to a boolean.
     *
     * Allows dragging of the switch's nub.
     *
     * The toggle behaves like any [AngularJS checkbox](http://docs.angularjs.org/api/ng/input/input[checkbox]) otherwise.
     *
     * @param toggle-class {string=} Sets the CSS class on the inner `label.toggle` element created by the directive.
     *
     * @usage
     * Below is an example of a toggle directive which is wired up to the `airplaneMode` model
     * and has the `toggle-calm` CSS class assigned to the inner element.
     *
     * ```html
     * <ior-toggle ng-model="airplaneMode" toggle-class="toggle-calm" toggle-text-off="OFF" toggle-text-on="ON">Airplane Mode</ior-toggle>
     * ```
     */
    .directive('iorToggle', [
        '$timeout',
        '$ionicConfig',
        function($timeout, $ionicConfig) {

            return {
                restrict: 'E',
                replace: true,
                require: '?ngModel',
                transclude: true,
                template:
                '<div class="item item-toggle">' +
                '<div ng-transclude></div>' +
                '<div>' +
                '<span class="label-off"></span>' +
                '<label class="toggle">' +
                '<input type="checkbox">' +
                '<div class="track">' +
                '<div class="handle"></div>' +
                '</div>' +
                '</label>' +
                '<span class="label-on"></span>' +
                '</div>' +
                '</div>',

                compile: function(element, attr) {
                    var input = element.find('input');
                    forEach({
                        'name': attr.name,
                        'ng-value': attr.ngValue,
                        'ng-model': attr.ngModel,
                        'ng-checked': attr.ngChecked,
                        'ng-disabled': attr.ngDisabled,
                        'ng-true-value': attr.ngTrueValue,
                        'ng-false-value': attr.ngFalseValue,
                        'ng-change': attr.ngChange
                    }, function(value, name) {
                        if (isDefined(value)) {
                            input.attr(name, value);
                        }
                    });

                    if (attr.toggleClass) {
                        element[0].getElementsByTagName('label')[0].classList.add(attr.toggleClass);
                    }
                    if (attr.toggleTextOff) {
                        element[0].getElementsByClassName('label-off')[0].innerText = attr.toggleTextOff;
                    }
                    if (attr.toggleTextOn) {
                        element[0].getElementsByClassName('label-on')[0].innerText = attr.toggleTextOn;
                    }

                    element.addClass('toggle-' + $ionicConfig.form.toggle());

                    return function($scope, $element) {
                        var el = $element[0].getElementsByTagName('label')[0];
                        var checkbox = el.children[0];
                        var track = el.children[1];
                        var handle = track.children[0];

                        var ngModelController = jqLite(checkbox).controller('ngModel');

                        $scope.toggle = new ionic.views.Toggle({
                            el: el,
                            track: track,
                            checkbox: checkbox,
                            handle: handle,
                            onChange: function() {
                                if (ngModelController) {
                                    ngModelController.$setViewValue(checkbox.checked);
                                    $scope.$apply();
                                }
                            }
                        });

                        $scope.$on('$destroy', function() {
                            $scope.toggle.destroy();
                        });
                    };
                }

            };
        }
    ])







    /**
     * @ngdoc directive
     * @name iorButtonGroup
     * @module ionic
     * @codepen tfAzj
     * @restrict E
     *
     * @description
     * A toggle is an animated switch which binds a given model to a boolean.
     *
     * Allows dragging of the switch's nub.
     *
     * The toggle behaves like any [AngularJS checkbox](http://docs.angularjs.org/api/ng/input/input[checkbox]) otherwise.
     *
     * @param toggle-class {string=} Sets the CSS class on the inner `label.toggle` element created by the directive.
     *
     * @usage
     * Below is an example of a toggle directive which is wired up to the `airplaneMode` model
     * and has the `toggle-calm` CSS class assigned to the inner element.
     *
     * ```html
     * <ior-button-group ng-model="airplaneMode" toggle-class="toggle-calm" toggle-text-off="OFF" toggle-text-on="ON">Airplane Mode</ior-button-group>
     * ```
     */
    .directive('iorToggle2', [
        '$timeout',
        '$ionicConfig',
        function($timeout, $ionicConfig) {

            return {
                restrict: 'E',
                replace: true,
                require: '?ngModel',
                scope: {
                    state: '=ngModel'
                },
                transclude: true,

                template:
                '<div class="item item-toggle2">' +
                    '<div ng-transclude></div>' +
                    '<div class="btn-group">' +
                        '<button class="button label-off" ng-class="{active: !isOn()}" ng-click="turnOff()"></button>' +
                        '<button class="button label-on" ng-class="{active: isOn()}" ng-click="turnOn()"></button>' +
                    '</div>' +
                '</div>',

                //controller: function($scope) {
                //    $scope.turnOff = function() {
                //        $scope.state = false;
                //    };
                //
                //    $scope.turnOn = function() {
                //        $scope.state = true;
                //    };
                //
                //    $scope.isOn = function() {
                //        return $scope.state;
                //    };
                //},

                link : function(scope, element, attr, ctrl){
                    //scope.updateModel = function(item) {
                    //    ctrl.$setViewValue(item);
                    //};

                    ctrl.$viewChangeListeners.push(function() {
                        scope.$eval(attr.ngChange);
                    });

                    scope.turnOff = function() {
                        scope.state = false;
                        ctrl.$setViewValue(scope.state);
                    };

                    scope.turnOn = function() {
                        scope.state = true;
                        ctrl.$setViewValue(scope.state);
                    };

                    scope.isOn = function() {
                        return scope.state;
                    };

                    var input = element.find('input');
                    forEach({
                        'name': attr.name,
                        'ng-value': attr.ngValue,
                        'ng-model': attr.ngModel,
                        'ng-checked': attr.ngChecked,
                        'ng-disabled': attr.ngDisabled,
                        'ng-true-value': attr.ngTrueValue,
                        'ng-false-value': attr.ngFalseValue,
                        'ng-change': attr.ngChange
                    }, function(value, name) {
                        if (isDefined(value)) {
                            input.attr(name, value);
                        }
                    });

                    if (attr.toggleTextOff) {
                        element[0].getElementsByClassName('label-off')[0].innerText = attr.toggleTextOff;
                    }
                    if (attr.toggleTextOn) {
                        element[0].getElementsByClassName('label-on')[0].innerText = attr.toggleTextOn;
                    }
                }

                //compile: function(element, attr) {
                //
                //    return {
                //        pre: function preLink( scope, element, attr, ctrl, transcludeFn ) {
                //            //scope.updateModel = function(item) {
                //            //    ctrl.$setViewValue(item);
                //            //};
                //            //ctrl.$viewChangeListeners.push(function() {
                //            //    scope.$eval(attr.ngChange);
                //            //});
                //        },
                //        post: function postLink( scope, element, attr, ctrl, transcludeFn ) {
                //
                //            var input = element.find('input');
                //            forEach({
                //                'name': attr.name,
                //                'ng-value': attr.ngValue,
                //                'ng-model': attr.ngModel,
                //                'ng-checked': attr.ngChecked,
                //                'ng-disabled': attr.ngDisabled,
                //                'ng-true-value': attr.ngTrueValue,
                //                'ng-false-value': attr.ngFalseValue,
                //                'ng-change': attr.ngChange
                //            }, function(value, name) {
                //                if (isDefined(value)) {
                //                    input.attr(name, value);
                //                }
                //            });
                //
                //            if (attr.toggleTextOff) {
                //                element[0].getElementsByClassName('label-off')[0].innerText = attr.toggleTextOff;
                //            }
                //            if (attr.toggleTextOn) {
                //                element[0].getElementsByClassName('label-on')[0].innerText = attr.toggleTextOn;
                //            }
                //
                //        }
                //    };
                //}

            };
        }
    ])

    .directive('rangeSlider', [function () {
        return {
            replace: true,
            restrict: 'E',
            require: 'ngModel',
            template: '<input type="range"/>',
            link: function (scope, element, attrs, ngModel) {
                var ngRangeMin;
                var ngRangeMax;
                var ngRangeStep;
                var value;

                function init() {
                    if (!angular.isDefined(attrs.ngRangeMin)) {
                        ngRangeMin = 0;
                    } else {
                        scope.$watch(attrs.ngRangeMin, function (newValue, oldValue, scope) {
                            if (angular.isDefined(newValue)) {
                                ngRangeMin = newValue;
                                setValue();
                            }
                        });
                    }
                    if (!angular.isDefined(attrs.ngRangeMax)) {
                        ngRangeMax = 100;
                    } else {
                        scope.$watch(attrs.ngRangeMax, function (newValue, oldValue, scope) {
                            if (angular.isDefined(newValue)) {
                                ngRangeMax = newValue;
                                setValue();
                            }
                        });
                    }
                    if (!angular.isDefined(attrs.ngRangeStep)) {
                        ngRangeStep = 1;
                    } else {
                        scope.$watch(attrs.ngRangeStep, function (newValue, oldValue, scope) {
                            if (angular.isDefined(newValue)) {
                                ngRangeStep = newValue;
                                setValue();
                            }
                        });
                    }
                    if (!angular.isDefined(ngModel)) {
                        value = 50;
                    } else {
                        scope.$watch(
                            function () {
                                return ngModel.$modelValue;
                            },
                            function (newValue, oldValue, scope) {
                                if (angular.isDefined(newValue)) {
                                    value = newValue;
                                    setValue();
                                }
                            }
                        );
                    }

                    if (!ngModel) {
                        return;
                    }
                    ngModel.$parsers.push(function (value) {
                        var val = Number(value);
                        if (val !== val) {
                            val = undefined;
                        }
                        return val;
                    });
                }

                function setValue() {
                    if (
                        angular.isDefined(ngRangeMin) &&
                        angular.isDefined(ngRangeMax) &&
                        angular.isDefined(ngRangeStep) &&
                        angular.isDefined(value)
                        ) {
                        element.attr("min", ngRangeMin);
                        element.attr("max", ngRangeMax);
                        element.attr("step", ngRangeStep);
                        element.val(value);
                    }
                }

                function read() {
                    if (angular.isDefined(ngModel)) {
                        ngModel.$setViewValue(value);
                    }
                }

                element.on('change', function () {
                    if (angular.isDefined(value) && (value != element.val())) {
                        value = element.val();
                        scope.$apply(read);
                    }
                });

                init();
            }
        }
    }]);

})(window.ionic);
