angular.module('iOnRoad.services', [])

    .factory('_DBHelper', function() {
        return {
            init: function() {
                if (db === null) {
                    db = window.sqlitePlugin.openDatabase({
                        name: "ionroad.db",
                        androidDatabaseImplementation: 2,
                        createFromLocation: 1
                    });
                }
            }
        };
    })

    .factory('_Utils', function(_Settings) {
        return {
            // Converts SQLite results to an array
            sql2array: function(list) {
                // Declerations
                var result,
                    index;

                // Initializations
                result = [];

                // Logic
                for(index = 0; index < list.length; index++) {
                    result.push(list.item(index));
                }

                return result;
            },

            // Returns the windows dimensions
            getWindowDimensions: function() {
                return {
                    width: window.innerWidth,
                    height: window.innerHeight
                };
            },

            getShortReadableTimestamp: function(date) {
                // Timestamp is somehow returns as string, we need to convert it to a number
                date = parseInt(date);

                var time = moment(date);

                return time.format("DD MMM. YYYY");
            },

            getReadableTimestamp: function(date) {
                // Timestamp is somehow returns as string, we need to convert it to a number
                date = parseInt(date);

                var time = moment(date);

                return time.format("DD MMM, YYYY HH:mm");
            },

            getTimestamp: function(date) {
                // Timestamp is somehow returns as string, we need to convert it to a number
                date = parseInt(date);

                var time = moment(date);

                return time.format("DD/MM/YYYY");
            },

            // Translates the drives' date to a more readable version
            getDriveDateText: function(date) {
                // Timestamp is somehow returns as string, we need to convert it to a number
                date = parseInt(date);

                var drive = moment(date);

                if (moment().diff(drive, "seconds") < 60) {
                    return moment().diff(drive, "minutes") + "s ago";
                }
                if (moment().diff(drive, "hours") < 1) {
                    return moment().diff(drive, "minutes") + "m ago";
                }
                if (moment().diff(drive, "hours") < 24) {
                    return moment().diff(drive, "hours") + " hrs ago";
                }
                if (moment().diff(drive, "hours") >= 24 && moment().diff(drive, "hours") <= 48) {
                    return "Yesterday";
                }
                if (moment().diff(drive, "weeks") < 1) {
                    return drive.format("ddd") + ".";
                }

                return drive.format("DD MMM") + ".";
            },

            // Translates the drives' time to a more readable version
            // @time - the drives' time
            // -returns the time in a more readable version
            getTime: function(time) {
                // Timestamp is somehow returns as string, we need to convert it to a number
                time = parseInt(time);

                time = moment(time);

                return time.format("HH:mm");
            },

            // Decides the right metric for the drives' time
            // @time - the drives' time
            // -returns the string 'am' or 'pm' according to the given time
            getTimeMetric: function(time) {
                // Timestamp is somehow returns as string, we need to convert it to a number
                time = parseInt(time);

                time = moment(time);
                if (time.hour() > 12)
                    return "pm";
                else
                    return "am";
            },

            // Translates the drives' distance to a more readable version
            // @distance - the drives' distance
            // -returns the distance in a more readable version
            getDistance: function(distance) {
                if (distance >= 10) {
                    return distance.toPrecision(3);
                } else if (distance >= 1) {
                    return distance.toPrecision(2);
                } else {
                    return distance.toPrecision(1) * 1000;
                }
            },

            // Decides the right metric for the drives' distance according to the settings file
            // -returns 'km', 'm' or 'mi', 'ft' depending on the settings
            // boolean @type, true=kilometers, false=miles
            getDistanceMetric: function(distance, type) {
                if (type) {
                    if (distance >= 1)
                        return "km";
                    else
                        return "m";
                } else {
                    if (distance >= 1)
                        return "mi";
                    else
                        return "ft";
                }
            },

            // Translates the drives' duration to a more readable version
            // @start - the drives' start time
            // @end - the drives' end time
            // -returns the duration in a more readable and trimmed version
            getDuration: function(start, end) {
                // Timestamp is somehow returns as string, we need to convert it to a number
                start = parseInt(start);
                end = parseInt(end);

                start = moment(start);
                end = moment(end);

                if (end.diff(start, "hours")) {
                    return (end.diff(start, "hours")).toFixed(1);
                } else if (end.diff(start, "minutes")) {
                    return (end.diff(start, "minutes")).toFixed(0);
                } else {
                    return (end.diff(start, "seconds")).toFixed(0);
                }
            },

            // Translates the drives' duration to a more readable version
            // @start - the drives' start time
            // @end - the drives' end time
            // -returns the duration in a more readable and trimmed version
            getDurationTime: function(duration) {
                duration = moment.duration(duration);

                if (duration.hours()) {
                    return (duration.hours()).toFixed(1);
                } else if (duration.minutes()) {
                    return duration.minutes().toFixed(0);
                } else {
                    return duration.seconds().toFixed(0);
                }
            },

            // Decides the right metric for the drives' duration
            // @start - the drives' start time
            // @end - the drives' end time
            // -returns the string 'hrs'
            getDurationMetric: function(start, end) {
                // Timestamp is somehow returns as string, we need to convert it to a number
                start = parseInt(start);
                end = parseInt(end);

                start = moment(start);
                end = moment(end);

                if (end.diff(start, "hours")) {
                    return "hr";
                } else if (end.diff(start, "minutes")) {
                    return "min";
                } else {
                    return "sec";
                }
            },

            // Decides the right metric for the drives' duration
            // @time - the drives' start time
            // -returns the string 'hrs'
            getDurationTimeMetric: function(duration) {
                duration = moment.duration(duration);

                if (duration.hours()) {
                    return "hr";
                } else if (duration.minutes()) {
                    return "min";
                } else {
                    return "sec";
                }
            }
        };
    })

    .factory('_Settings', function() {
        return {
            initialized: function() {
                if (typeof sharedpreferences === "undefined") {
                    console.log("Failed to initialize sharedpreferences");
                    return false;
                } else {
                    console.log("Successfully initialized sharedpreferences");
                    return true;
                }
            },

            load: function() {
                sharedpreferences.getSharedPreferences("settings", "MODE_PRIVATE",
                    function (result) { console.log("Successfully opened the settings file: " + result); },
                    function (result) { console.log("Error opening the settings file: " + result); });

            },

            getValue: function(key, type, successHandler, errorHandler) {
                switch(type){
                    case "array":
                        sharedpreferences.getString(key, function(value) {
                            successHandler(value.replace('[','').replace(']','').split(", "));
                        }, errorHandler);
                        break;
                    case "string":
                        sharedpreferences.getString(key, successHandler, errorHandler);
                        break;
                    case "int":
                        sharedpreferences.getInt(key, successHandler, errorHandler);
                        break;
                    case "long":
                        sharedpreferences.getLong(key, successHandler, errorHandler);
                        break;
                    case "boolean":
                        sharedpreferences.getBoolean(key, function(value) {
                            successHandler(value==1);
                        }, errorHandler);
                        break;
                }
            },

            putValue: function(key, value, type, successHandler, errorHandler) {
                switch(type){
                    case "string":
                        sharedpreferences.putString(key, value, successHandler, errorHandler);
                        break;
                    case "int":
                        sharedpreferences.putInt(key, value, successHandler, errorHandler);
                        break;
                    case "long":
                        sharedpreferences.putLong(key, value, successHandler, errorHandler);
                        break;
                    case "boolean":
                        sharedpreferences.putBoolean(key, value, successHandler, errorHandler);
                        break;
                }
            }
        };
    })

    .factory('$localStorage', ['$window', function($window) {
        return {
            set: function(key, value) {
                $window.localStorage[key] = value;
            },
            get: function(key, defaultValue) {
                return $window.localStorage[key] || defaultValue;
            },
            setObject: function(key, value) {
                $window.localStorage[key] = JSON.stringify(value);
            },
            getObject: function(key) {
                return JSON.parse($window.localStorage[key] || '{}');
            }
        };
    }])

    /* ===================================================
     * RoutesService
     * =================================================== */
    .factory("RoutesService", function(_Utils, _DBHelper) {
        return {
            run: function (runId, callback) {
                try {
                    // Initialize the DB if not initialized already
                    _DBHelper.init();

                    db.executeSql(
                        "SELECT RunId, Timestamp, Latitude, Longitude " +
                        "FROM Routes " +
                        "WHERE RunId=? " +
                        "ORDER BY Timestamp ASC", [runId], function (res) {
                            return callback(_Utils.sql2array(res.rows));
                        });
                }
                catch (e) {
                    console.log(e.toString());
                    return callback([]);
                }
            },
        };
    })

    /* ===================================================
     * RunsService
     * =================================================== */
    .factory("RunsService", function(_Utils, _DBHelper, $q) {
        return {
            clear_local_data: function() {
                try {
                    // Initialize the DB if not initialized already
                    _DBHelper.init();

                    var promises = [];

                    var q_fcw_alerts = $q.defer();
                    promises.push(q_fcw_alerts);
                    db.executeSql(
                        "DELETE FROM FCWAlerts", [], function (res) {
                            q_fcw_alerts.resolve(true);
                        });

                    var q_speeding_alerts = $q.defer();
                    promises.push(q_speeding_alerts);
                    db.executeSql(
                        "DELETE FROM SpeedingAlerts", [], function (res) {
                            q_speeding_alerts.resolve(true);
                        });

                    var q_ldw_alerts = $q.defer();
                    promises.push(q_ldw_alerts);
                    db.executeSql(
                        "DELETE FROM LDWAlerts", [], function (res) {
                            q_ldw_alerts.resolve(true);
                        });

                    var q_routes_alerts = $q.defer();
                    promises.push(q_routes_alerts);
                    db.executeSql(
                        "DELETE FROM Routes", [], function (res) {
                            q_routes_alerts.resolve(true);
                        });

                    var q_runs_alerts = $q.defer();
                    promises.push(q_runs_alerts);
                    db.executeSql(
                        "DELETE FROM Runs", [], function (res) {
                            q_runs_alerts.resolve(true);
                        });

                    var q_vaccum = $q.defer();
                    promises.push(q_vaccum);
                    db.executeSql(
                        "VACUUM", [], function (res) {
                            q_vaccum.resolve(true);
                        });

                    return $q.all(promises);
                }
                catch (e) {
                    console.log(e.toString());
                    return false;
                }
            },

            all: function (callback) {
                try {
                    // Initialize the DB if not initialized already
                    _DBHelper.init();

                    db.executeSql(
                        "SELECT RunId, StartTime, EndTime, TotalDistance " +
                        "FROM Runs ", [], function (res) {
                            return callback(_Utils.sql2array(res.rows));
                        });
                }
                catch (e) {
                    console.log(e.toString());
                    return callback([]);
                }
            },

            count: function (callback) {
                try {
                    // Initialize the DB if not initialized already
                    _DBHelper.init();

                    db.executeSql(
                        "SELECT count(*) as count " +
                        "FROM Runs ", [], function (res) {
                            if (res.rows.length > 0) {
                                return callback(res.rows.item(0).count);
                            }
                            else {
                                return callback(0);
                            }
                        });
                }
                catch (e) {
                    console.log(e.toString());
                    return callback(0);
                }
            },

            run: function (runId, callback) {
                try {
                    // Initialize the DB if not initialized already
                    _DBHelper.init();

                    db.executeSql(
                        "SELECT RunId, StartTime, EndTime, TotalDistance " +
                        "FROM Runs " +
                        "WHERE RunId=? " +
                        "ORDER BY RunId DESC", [runId], function (res) {
                            if (res.rows.length > 0) {
                                return callback(res.rows.item(0));
                            }
                            else {
                                return callback(null);
                            }
                        });
                }
                catch (e) {
                    console.log(e.toString());
                    return callback([]);
                }
            },

            latest: function (count, callback) {
                try {
                    // Initialize the DB if not initialized already
                    _DBHelper.init();

                    db.executeSql(
                        "SELECT RunId, StartTime, EndTime, TotalDistance " +
                        "FROM Runs " +
                        "ORDER BY RunId DESC " +
                        "LIMIT ?", [count], function (res) {
                            return callback(_Utils.sql2array(res.rows));
                        });
                }
                catch (e) {
                    console.log(e.toString());
                    return callback([]);
                }
            },

            delete: function (runId) {
                try {
                    // Initialize the DB if not initialized already
                    _DBHelper.init();

                    var promises = [];

                    var q_fcw_alerts = $q.defer();
                    promises.push(q_fcw_alerts);
                    db.executeSql(
                        "DELETE FROM FCWAlerts " +
                        "WHERE RunId=?", [runId], function (res) {
                            q_fcw_alerts.resolve(true);
                        });

                    var q_speeding_alerts = $q.defer();
                    promises.push(q_speeding_alerts);
                    db.executeSql(
                        "DELETE FROM SpeedingAlerts " +
                        "WHERE RunId=?", [runId], function (res) {
                            q_speeding_alerts.resolve(true);
                        });

                    var q_ldw_alerts = $q.defer();
                    promises.push(q_ldw_alerts);
                    db.executeSql(
                        "DELETE FROM LDWAlerts " +
                        "WHERE RunId=?", [runId], function (res) {
                            q_ldw_alerts.resolve(true);
                        });

                    var q_routes_alerts = $q.defer();
                    promises.push(q_routes_alerts);
                    db.executeSql(
                        "DELETE FROM Routes " +
                        "WHERE RunId=?", [runId], function (res) {
                            q_routes_alerts.resolve(true);
                        });

                    var q_runs_alerts = $q.defer();
                    promises.push(q_runs_alerts);
                    db.executeSql(
                        "DELETE FROM Runs " +
                        "WHERE RunId=?", [runId], function (res) {
                            q_runs_alerts.resolve(true);
                        });

                    var q_vaccum = $q.defer();
                    promises.push(q_vaccum);
                    db.executeSql(
                        "VACUUM", [], function (res) {
                            q_vaccum.resolve(true);
                        });

                    return $q.all(promises);
                }
                catch (e) {
                    console.log(e.toString());
                    return false;
                }
            }
        };
    })

    /* ===================================================
     * UsersService
     * =================================================== */
    .factory("UsersService", function(_DBHelper) {
        return {
            setUserInfo: function(type, email, picture, callback) {
                try {
                    // Initialize the DB if not initialized already
                    _DBHelper.init();

                    db.executeSql(
                        "UPDATE Users " +
                        "SET AccountType=?, AccountName=?, ProfilePic=?", [type, email, picture], function (res) {
                            return callback();
                        });
                }
                catch (e) {
                    console.log(e.toString());
                    return callback([]);
                }
            },

            getUserInfo: function(callback) {
                try {
                    // Initialize the DB if not initialized already
                    _DBHelper.init();

                    db.executeSql(
                        "SELECT * " +
                        "FROM Users", [], function (res) {
                            if (res.rows.length > 0) {
                                return callback(res.rows.item(0));
                            }
                            else {
                                return callback(null);
                            }
                        });
                }
                catch (e) {
                    console.log(e.toString());
                    return callback([]);
                }
            },

            isUserLoggedIn: function(callback) {
                try {
                    // Initialize the DB if not initialized already
                    _DBHelper.init();

                    db.executeSql(
                        "SELECT AccountName " +
                        "FROM Users", [], function (res) {
                            if (res.rows.length > 0) {
                                var user = res.rows.item(0);
                                if (user.AccountName) {
                                    return callback(true);
                                }
                                else {
                                    return callback(false);
                                }
                            }
                            else {
                                return callback(false);
                            }
                        });
                }
                catch (e) {
                    console.log(e.toString());
                    return callback([]);
                }
            },
        };
    })

    .factory("HomeService", function(RunsService, UsersService) {
        return {
            getUserInfo: function(callback) {
                return UsersService.getUserInfo(callback);
            },

            getLatestRuns: function(callback) {
                return RunsService.latest(3, callback);
            }
        };
    })

    .factory("MyDrivesService", function(RunsService, _Utils, _DBHelper) {
        return {
            getAllRuns: function(callback) {
                try {
                    // Initialize the DB if not initialized already
                    _DBHelper.init();

                    db.executeSql(
                        "SELECT RunId, StartTime, EndTime, TotalDistance " +
                        "FROM Runs " +
                        "ORDER BY RunId DESC ", [], function (res) {
                            return callback(_Utils.sql2array(res.rows));
                        });
                }
                catch (e) {
                    console.log(e.toString());
                    return callback([]);
                }
            },

            getTotalDistance: function(callback) {
                try {
                    // Initialize the DB if not initialized already
                    _DBHelper.init();

                    db.executeSql(
                        "SELECT SUM(TotalDistance) as distance " +
                        "FROM Runs ", [], function (res) {
                            if (res.rows.length > 0) {
                                return callback(parseFloat(res.rows.item(0).distance.toFixed(2)));
                            }
                            else {
                                return callback(null);
                            }
                        });
                }
                catch (e) {
                    console.log(e.toString());
                    return callback([]);
                }
            },

            getTotalTime: function(callback) {
                try {
                    // Initialize the DB if not initialized already
                    _DBHelper.init();

                    db.executeSql(
                        "SELECT StartTime, EndTime " +
                        "FROM Runs ", [], function (res) {
                            var result = _Utils.sql2array(res.rows);
                            var totalTime = 0;
                            for (i = 0; i < result.length; i++) {
                                var startTime = moment(parseInt(result[i].StartTime));
                                var endTime = moment(parseInt(result[i].EndTime));
                                totalTime += (endTime.diff(startTime));
                            }
                            return callback(totalTime);
                        });
                }
                catch (e) {
                    console.log(e.toString());
                    return callback([]);
                }
            },

            getTotalAlerts: function(callback) {
                try {
                    // Initialize the DB if not initialized already
                    _DBHelper.init();

                    db.executeSql(
                        "SELECT " +
                        "( " +
                        "SELECT count(*) " +
                        "FROM   FCWAlerts " +
                        ") " +
                        //"( " +
                        //"SELECT count(*) " +
                        //"FROM   LDWAlerts " +
                        //") + " +
                        //"( " +
                        //"SELECT count(*) "+
                        //"FROM   SpeedingAlerts " +
                        //")" +
                        "as alerts", [], function (res) {
                            if (res.rows.length > 0) {
                                return callback(res.rows.item(0).alerts);
                            }
                            else {
                                return 0;
                            }
                        });
                }
                catch (e) {
                    console.log(e.toString());
                    return callback([]);
                }
            }
        };
    })

    .factory("DriveService", function(RunsService, RoutesService, _Utils, _DBHelper) {
        return {
            getRun: function(runId, callback) {
                return RunsService.run(runId, callback);
            },

            getRoute: function(runId, callback) {
                return RoutesService.run(runId, callback);
            },

            getRunDistance: function(runId, callback) {
                try {
                    // Initialize the DB if not initialized already
                    _DBHelper.init();

                    db.executeSql(
                        "SELECT TotalDistance " +
                        "FROM Runs " +
                        "WHERE RunId=? ", [runId], function (res) {
                            if (res.rows.length > 0) {
                                return callback(res.rows.item(0));
                            }
                            else {
                                return callback(null);
                            }
                        });
                }
                catch (e) {
                    console.log(e.toString());
                    return callback([]);
                }
            },

            getAlerts: function(runId, threshold, callback) {
                try {
                    var collisionAlerts, offroadAlerts, tailgatingAlerts, speedingAlerts, timer;
                    collisionAlerts = null;
                    offroadAlerts = null;
                    tailgatingAlerts = null;
                    speedingAlerts = null;
                    this.getCollisionAlerts(runId, function(result) {
                        collisionAlerts = result;
                    });
                    this.getOffroadAlerts(runId, function(result) {
                        offroadAlerts = result;
                    });
                    this.getTailgatingAlerts(runId, threshold, function(result) {
                        tailgatingAlerts = result;
                    });
                    this.getSpeedingAlerts(runId, function(result) {
                        speedingAlerts = result;
                    });

                    timer = setInterval(function() {
                        if (collisionAlerts && offroadAlerts && tailgatingAlerts && speedingAlerts) {
                            callback({
                                collision: collisionAlerts,
                                offroad: offroadAlerts,
                                tailgating: tailgatingAlerts,
                                speeding: speedingAlerts
                            });
                            clearInterval(timer);
                        }
                    }, 100);
                }
                catch (e) {
                    console.log(e.toString());
                    return callback([]);
                }
            },

            getCollisionAlerts: function(runId, callback) {
                try {
                    // Initialize the DB if not initialized already
                    _DBHelper.init();

                    db.executeSql(
                        "SELECT RunId, Timestamp, Latitude, Longitude " +
                        "FROM FCWAlerts " +
                        "WHERE RunId=?", [runId], function (res) {
                            return callback(_Utils.sql2array(res.rows));
                        });
                }
                catch (e) {
                    console.log(e.toString());
                    return callback([]);
                }
            },

            getOffroadAlerts: function(runId, callback) {
                try {
                    // Initialize the DB if not initialized already
                    _DBHelper.init();

                    db.executeSql(
                        "SELECT RunId, Timestamp, Latitude, Longitude " +
                        "FROM LDWAlerts " +
                        "WHERE RunId=?", [runId], function (res) {
                            return callback(_Utils.sql2array(res.rows));
                        });
                }
                catch (e) {
                    console.log(e.toString());
                    return callback([]);
                }
            },

            getTailgatingAlerts: function(runId, threshold, callback) {
                try {
                    // Initialize the DB if not initialized already
                    _DBHelper.init();

                    db.executeSql(
                        "SELECT * " +
                        "FROM Routes " +
                        "WHERE RunId=?", [runId], function (res) {
                            var coors, result, inTailgating;
                            coors = _Utils.sql2array(res.rows);
                            result = [];
                            inTailgating = false;
                            for (i=0; i < coors.length; i++) {
                                var coor, distance, speed, time_gap;
                                c = coors[i];
                                distance = c.TailgatingDistance;
                                speed = c.Speed / 3.6;
                                time_gap = distance / speed;

                                if (distance === 0)
                                {
                                    // none;
                                    result[result.length] = {
                                        coor: c,
                                        state: 0
                                    };
                                }
                                else if (time_gap <= threshold)
                                {
                                    // red;
                                    result[result.length] = {
                                        coor: c,
                                        state: 1
                                    };
                                }
                                else if(time_gap <= threshold + 0.3)
                                {
                                    // yellow;
                                    result[result.length] = {
                                        coor: c,
                                        state: 2
                                    };
                                }
                                else
                                {
                                    // green;
                                   result[result.length] = {
                                        coor: c,
                                        state: 3
                                    };
                                }
                            }
                            return callback(result);
                        });
                }
                catch (e) {
                    console.log(e.toString());
                    return callback([]);
                }
            },

            getSpeedingAlerts: function(runId, callback) {
                try {
                    // Initialize the DB if not initialized already
                    _DBHelper.init();

                    db.executeSql(
                        "SELECT RunId, Timestamp, Latitude, Longitude, SpeedSign " +
                        "FROM SpeedingAlerts " +
                        "WHERE RunId=?", [runId], function (res) {
                            return callback(_Utils.sql2array(res.rows));
                        });
                }
                catch (e) {
                    console.log(e.toString());
                    return callback([]);
                }
            },
        };
    })

    .factory("ParkingService", function(_Utils, _DBHelper) {
        return {
            // getAlarm: function(callback) {
            //     try {
            //         // Initialize the DB if not initialized already
            //         _DBHelper.init();
            //
            //         db.executeSql(
            //             "SELECT ReminderTime " +
            //             "FROM ParkingInfo", [], function (res) {
            //                 if (res.rows.length > 0) {
            //                     return callback(res.rows.item(0));
            //                 }
            //                 else {
            //                     return callback(null);
            //                 }
            //             });
            //     }
            //     catch (e) {
            //         console.log(e.toString());
            //         return callback([]);
            //     }
            // },

            setAlarm: function(timestamp, callback) {
                try {
                    // Initialize the DB if not initialized already
                    _DBHelper.init();

                    db.executeSql(
                        "UPDATE ParkingInfo " +
                        "SET ReminderTime=?", [timestamp], function (res) {
                            return callback();
                        });
                }
                catch (e) {
                    console.log(e.toString());
                    return callback([]);
                }
            },

            cancelAlarm: function(callback) {
                try {
                    // Initialize the DB if not initialized already
                    _DBHelper.init();

                    db.executeSql(
                        "UPDATE ParkingInfo " +
                        "SET ReminderTime=?", [null], function (res) {
                            return callback();
                        });
                }
                catch (e) {
                    console.log(e.toString());
                    return callback([]);
                }
            },

            setParkingLocation: function(timestamp, long, lat, country, city, address, callback) {
                try {
                    // Initialize the DB if not initialized already
                    _DBHelper.init();

                    db.executeSql(
                        "UPDATE ParkingInfo " +
                        "SET Timestamp=?, Longitude=?, Latitude=?, Altitude=?, Country=?, City=?, Address=?, State=?, ParkingSnapshot=?", [timestamp, long, lat, null, country, city, address, null, null], function (res) {
                            return callback();
                        });
                }
                catch (e) {
                    console.log(e.toString());
                    return callback([]);
                }
            },

            setParkingPhoto: function(photo, callback) {
                try {
                    // Initialize the DB if not initialized already
                    _DBHelper.init();

                    db.executeSql(
                        "UPDATE ParkingInfo " +
                        "SET ParkingSnapshot=?", [photo], function (res) {
                            return callback();
                        });
                }
                catch (e) {
                    console.log(e.toString());
                    return callback([]);
                }
            },

            isParkingExists: function(callback) {
                try {
                    // Initialize the DB if not initialized already
                    _DBHelper.init();

                    db.executeSql(
                        "SELECT RunId " +
                        "FROM ParkingInfo", [], function (res) {
                            if (res.rows.length > 0) {
                                return callback(true);
                            }
                            else {
                                return callback(false);
                            }
                        });
                }
                catch (e) {
                    console.log(e.toString());
                    return callback([]);
                }
            },

            getParkingPhoto: function(callback) {
                try {
                    // Initialize the DB if not initialized already
                    _DBHelper.init();

                    db.executeSql(
                        "SELECT ParkingSnapshot " +
                        "FROM ParkingInfo", [], function (res) {
                            if (res.rows.length > 0) {
                                return callback(res.rows.item(0).ParkingSnapshot);
                            }
                            else {
                                return callback(null);
                            }
                        });
                }
                catch (e) {
                    console.log(e.toString());
                    return callback([]);
                }
            },

            getParkingInfo: function(callback) {
                try {
                    // Initialize the DB if not initialized already
                    _DBHelper.init();

                    db.executeSql(
                        "SELECT * " +
                        "FROM ParkingInfo", [], function (res) {
                            if (res.rows.length > 0) {
                                return callback(res.rows.item(0));
                            }
                            else {
                                return callback(null);
                            }
                        });
                }
                catch (e) {
                    console.log(e.toString());
                    return callback([]);
                }
            },

            getReminderInfo: function(callback) {
                try {
                    // Initialize the DB if not initialized already
                    _DBHelper.init();

                    db.executeSql(
                        "SELECT ReminderTime " +
                        "FROM ParkingInfo", [], function (res) {
                            if (res.rows.length > 0) {
                                return callback(res.rows.item(0));
                            }
                            else {
                                return callback(null);
                            }
                        });
                }
                catch (e) {
                    console.log(e.toString());
                    return callback([]);
                }
            },
        };
    });
